#
# A simple sensitivity analysis, with Sobol indices.
# "Introduction to sensitivity analysis with NISP
# Michael Baudin (EDF), Jean-Marc Martinez (CEA)"
#
# EDF R&D - 2012-2013 - Michael Baudin
#

from math import sin, pi
from openturns import JointDistribution, Uniform, SymbolicFunction


def ishigamiAB():
    a = 7.0
    b = 0.1
    return [a, b]


# Calculs exacts
def ishigamiSAExact(a, b):
    meanY = a / 2.0
    varY = 1.0 / 2.0 + a**2 / 8 + b * pi**4 / 5 + b**2 * pi**8 / 18
    S1 = (1.0 / 2.0 + b * pi**4 / 5 + b**2 * pi**8 / 50) / varY
    S2 = (a**2 / 8) / varY
    S3 = 0
    # S12 = 0;
    # S23 = 0;
    S13 = b**2 * pi**8 / 2.0 * (1.0 / 9.0 - 1.0 / 25.0) / varY
    # S123 = 0;
    ST1 = S1 + S13
    ST2 = S2
    ST3 = S3 + S13
    return [meanY, varY, S1, S2, S3, ST1, ST2, ST3]


# Define the model as a Python function
# Ishigami
def ishigamiG(x):
    a, b = ishigamiAB()
    y = sin(x[0]) + a * sin(x[1]) ** 2 + b * x[2] ** 4 * sin(x[0])
    return [y]


def ishigamiGSymbolic():
    formula = ["sin(X1)+7*sin(X2)*sin(X2)+0.1*X3*X3*X3*X3*sin(X1)"]
    model = SymbolicFunction(["X1", "X2", "X3"], formula)
    return model


def ishigamiDistribution():
    distribution = JointDistribution([Uniform(-pi, pi)] * 3)
    return distribution


if __name__ == "__main__":
    # Number of input parameters
    dim = 3

    # Total degree
    totdeg = 9

    # Definition des constantes
    a, b = ishigamiAB()

    [meanY, varY, S1, S2, S3, ST1, ST2, ST3] = ishigamiSAExact(a, b)

    print("Exact Results:")
    print("E(Y):", meanY)
    print("V(Y):", varY)
    print("S:", S1, S2, S3)
    print("ST:", ST1, ST2, ST3)

    #   expectation: 3.5
    #   var: 13.844588
    #   S: 0.3139052, 0.4424111, 0
    #   ST: 0.5575889, 0.4424111, 0.2436837



import openturns as ot
from matplotlib import pyplot as plt
import openturns.viewer as otv

# Create the mesher
mesher = ot.LevelSetMesher([50] * 2)

# Create a level set
function = ot.SymbolicFunction(["x0", "x1"], ["10*(x0^3+x1)^2+x0^2"])
level = 0.5
set = ot.LevelSet(function, ot.LessOrEqual(), level)

# Mesh the level set
mesh = mesher.build(set, ot.Interval([-1.0] * 2, [1.0] * 2))

# Draw the first mesh
graph = mesh.draw()
graph.setXTitle("$x_0$")
graph.setYTitle("$x_1$")
graph.setTitle("Mesh of a level set")

fig = plt.figure(figsize=(10, 4))
graph_axis = fig.add_subplot(111)
graph_axis.set_xlim(auto=True)

otv.View(graph, figure=fig, axes=[graph_axis], add_legend=True)



"""
Estimate a GPD on the Wooster temperature data
==============================================
"""

# %%
# In this example, we illustrate various techniques of extreme value modeling applied
# to the 5-year series of daily minimum temperatures recorded in Wooster, Ohio between 1983 and 1988.
# Readers should refer to [coles2001]_ example 1.7 to get more details.
import openturns as ot
import openturns.viewer as otv
from openturns.usecases import coles
import pandas as pd

# %%
# First, we load the Wooster dataset. As we want to model very low temperatures,
# we first negate the values to transform minimum to maximum. Hence, large positive
# observations correspond to extreme cold conditions. We look at the negated data
# through time. The data are plotted as degrees Fahrenheit below zero. We see that
# there is a strong annual cycle in the data.
#
# We use the pandas library.
full = pd.read_csv(coles.Coles().wooster, index_col=0, parse_dates=True)
full["Temperature"] = full["Temperature"].apply(lambda x: x * -1.0)
print(full[:10])
title = "Negated Wooster daily minimum temperatures"
graph = ot.Graph(title, "Day index", "Temperature (°F)")
days = ot.Sample([[i] for i in range(len(full))])
sample = ot.Sample.BuildFromDataFrame(full)
cloud = ot.Cloud(days, sample)
cloud.setColor("red")
cloud.setPointStyle(",")
graph.add(cloud)
graph.setIntegerXTick(True)
view = otv.View(graph)

# %%
# In order to decrease the time-varying trend, we partition the data into the four
# seasons. To perform that data stratification, we use the pandas library once again.
# The graphs show that there is still non-stationarity within each season's data
# but to a lesser extent than in the original series.
full_season = {}
full_season["winter"] = full[(full.index.month >= 12) | (full.index.month < 3)]
full_season["spring"] = full[(full.index.month >= 3) & (full.index.month < 6)]
full_season["summer"] = full[(full.index.month >= 6) & (full.index.month < 9)]
full_season["autumn"] = full[(full.index.month >= 9) & (full.index.month < 12)]
for season in full_season:
    df = full_season[season]
    days = ot.Sample([[i] for i in range(len(df))])
    sample = ot.Sample.BuildFromDataFrame(df)
    cloud = ot.Cloud(days, sample)
    cloud.setColor("red")
    cloud.setPointStyle(",")
    graph.setDrawable(0, cloud)
    graph.setTitle(f"{title} ({season})")
    view = otv.View(graph)

# %%
# Here, we illustrate that threshold exceedances occur in groups: one
# extremely cold day is likely to be followed by another, implying that observations
# exceeding a high threshold are dependent. Hence we use the declustering method
# which filters the dependent observations exceeding a given threshold to obtain a
# set of threshold excesses that can be assumed as independent.
#
# First, we specify a threshold :math:`u`.
# Consecutive exceedances of the threshold belong to the same cluster. Two distinct
# clusters are separated by :math:`r` consecutive observations under the
# threshold. Within each cluster, we select the maximum value that will be used to
# infer the GPD distribution. The cluster maxima are assumed to be independent.
#
# On the graph, we show the clusters associated to the threshold :math:`u=0`
# and the respective maximum selected from a
# 100-day portion of the Wooster daily minimum temperature series.
# It is possible to extract the data belonging to the same cluster and the
# cluster maximum series.
first100 = ot.Sample.BuildFromDataFrame(full[350:450])
part = ot.SamplePartition(first100)
u = 0.0
r = 4
peaks, clusters = part.getPeakOverThreshold(u, r)
graph = clusters.draw(u)
view = otv.View(graph)

# %%
# Here, we illustrate the effect of different choices for :math:`u`
# and :math:`r` on the estimate of the GPD distriution. We focus on the winter
# season. We perform the following steps, for each :math:`(u, r)`:
#
# - we extract the clusters and the associated peaks,
# - we fit a GPD distribution on the excesses by the maximum likelihood method,
# - we estimate the :math:`95\%` confidence interval of each parameter,
# - we evaluate the :math:`T=100`-year return level which corresponds to the
#   :math:`m`-observation return level, where :math:`m = T*n_y` with :math:`n_y`
#   the number of observations per year. Here, we consider the winter season
#   which is about 90 days long. To calculate it, we evaluate the extremal index
#   :math:`\theta` which is the inverse of the mean length of the clusters,
#   estimated by the ratio between the number of clusters and the number
#   of exceedances of :math:`u`.
#
# The stability in the return level estimations confirms that the
# inference is robust despite the subjective choices that need to be made
# on :math:`(u, r)`.
winter = full_season["winter"]
winter_sample = ot.Sample.BuildFromDataFrame(winter)

# partition the aggregated winter sample according to indices (enforces separation of seasons from different years)
part = ot.SamplePartition.ExtractFromDataFrame(full, winter)

factory = ot.GeneralizedParetoFactory()
for u in [-10.0, -20.0]:
    for r in [1, 3]:
        peaks, clusters = part.getPeakOverThreshold(u, r)
        nc = len(peaks)
        nu = (winter > u).values.sum()

        # fit a stationary gpd on the clusters
        result_LL = factory.buildMethodOfLikelihoodMaximizationEstimator(peaks, u)
        sigma, xi, _ = result_LL.getParameterDistribution().getMean()
        (
            sigma_stddev,
            xi_stddev,
            _,
        ) = result_LL.getParameterDistribution().getStandardDeviation()
        print(
            f"u={u} r={r} nc={nc} sigma={sigma:.2f} ({sigma_stddev:.2f}) xi={xi:.2f} ({xi_stddev:.2f})",
            end=" ",
        )

        # estimate the T-year return level
        ny = 90
        T = 100
        theta = nc / nu
        xm_100 = factory.buildReturnLevelEstimator(
            result_LL, winter_sample, T * ny, theta
        )
        print(f"x100={xm_100.getMean()} ({xm_100.getStandardDeviation()})")

        # plot the return level
        validation = ot.GeneralizedParetoValidation(result_LL, peaks)
        grid = validation.drawDiagnosticPlot()
        rlPlot = grid.getGraph(1, 0)
        rlPlot.setTitle(rlPlot.getTitle() + f" (u={u} r={r})")
        view = otv.View(rlPlot)

# %%
otv.View.ShowAll()



"""
Vertex value function
=====================
"""

# %%
#
# A vertex value function
# :math:`f_{vertexvalue}: \mathcal{D} \times \mathbb{R}^d \rightarrow \mathcal{D} \times \mathbb{R}^q` is a
# particular field function that lets invariant the mesh of a field
# and defined by a function :math:`h : \mathbb{R}^n \times \mathbb{R}^d  \rightarrow \mathbb{R}^q` such that:
#
# .. math::
#    \begin{aligned} f_{vertexvalue}(\vect{t}, \vect{x})=(\vect{t}, h(\vect{t},\vect{x}))\end{aligned}
#
# Let's note that the input dimension of :math:`f_{vertexvalue}` still design the
# dimension of :math:`\vect{x}` : :math:`d`. Its output dimension is equal to :math:`q`.
#
# The creation of the *VertexValueFunction* object requires the
# function :math:`h` and the integer :math:`n`: the dimension of the
# vertices of the mesh :math:`\mathcal{M}`.
#
# This example illustrates the creation of a field from the function
# :math:`h:\mathbb{R}\times\mathbb{R}^2` such as:
#
# .. math::
#    \begin{aligned}
#       h(\vect{t}, \vect{x})=(t+x_1^2+x_2^2)
#    \end{aligned}
#

# %%
import openturns as ot


# %%
# Create a mesh
N = 100
mesh = ot.RegularGrid(0.0, 1.0, N)

# %%
# Create the function that acts the values of the mesh
h = ot.SymbolicFunction(["t", "x1", "x2"], ["t+x1^2+x2^2"])

# %%
# Create the field function
f = ot.VertexValueFunction(h, mesh)

# %%
# Evaluate f
inF = ot.Normal(2).getSample(N)
outF = f(inF)

# print input/output at first 10 mesh nodes
txy = mesh.getVertices()
txy.stack(inF)
txy.stack(outF)
txy[:10]



# %%
import os
import openturns as ot


class FissionGasRelease:
    r"""
    Data class for the fission gas release example.

    Attributes
    ----------

    measurement_values : :class:`~openturns.Point`
        Observed values :math:`y_i` (:math:`1 \leq i \leq \sampleSize_{\mathrm{exp}}`)
        taken from the International Fuel Performance Experiments (IFPE) database

    measurement_uncertainty : :class:`~openturns.Function`
        Function :math:`y_i \mapsto \sigma_{y_i}`

    models : list of :class:`~openturns.Function`
        List of the :math:`\model_i` models (:math:`1 \leq i \leq \sampleSize_{\mathrm{exp}}`)

    Examples
    --------
    >>> from openturns.usecases import fission_gas
    >>> # Load the fission gas release models
    >>> fgr = fission_gas.FissionGasRelease()
    """

    def __init__(self) -> None:
        current_dir = os.path.dirname(os.path.abspath(__file__))
        self._Xtrain = ot.Sample.ImportFromTextFile(
            os.path.join(current_dir, "fission_gas_Xtrain.csv"), ";"
        )
        self._ytrain = ot.Sample.ImportFromTextFile(
            os.path.join(current_dir, "fission_gas_ytrain.csv"), ";"
        )
        self._hyperparameters = ot.Sample.ImportFromTextFile(
            os.path.join(current_dir, "fission_gas_GPR_hyperparameters.csv"), ";"
        )

        self.models = []
        covariance = ot.SquaredExponential(self._Xtrain.getDimension())
        basis = ot.ConstantBasisFactory(self._Xtrain.getDimension()).build()
        for num, hyper in enumerate(self._hyperparameters):
            covariance.setFullParameter(hyper)
            gpr = ot.KrigingAlgorithm(
                self._Xtrain, self._ytrain.getMarginal(num), covariance, basis
            )
            gpr.setOptimizeParameters(False)
            gpr.run()
            self.models.append(gpr.getResult().getMetaModel())

        # Measurement array
        self.measurement_values = ot.Point(
            [
                0.228,
                0.015,
                0.265,
                0.253,
                0.016,
                0.173,
                0.13,
                0.067,
                0.048,
                0.018,
                0.104,
                0.066,
                0.16,
                0.022,
                0.095,
                0.035,
                0.321,
                0.035,
                0.009,
                0.037,
                0.05,
                0.28,
                0.136,
                0.221,
                0.296,
                0.13,
                0.085,
                0.058,
                0.449,
                0.029,
                0.018,
            ]
        )

        self.measurement_uncertainty = lambda v: ((v * 0.05) ** 2 + 1e-4) ** 0.5

    def getInputDescription(self):
        return self._Xtrain.getDescription()



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()


def sortNodesAndWeights(nodes, weights):
    """
    Sort nodes and weights of an Experiment.

    Parameters
    ----------
    nodes : ot.Sample(size, dimension)
        The sorted nodes.
    weights : ot.Point(size)
        The weights.

    Returns
    -------
    sortedNodes : ot.Sample(size, dimension)
        The nodes.
    sortedWeights : ot.Point(size)
        The sorted weights.

    """
    indices = nodes.argsort()
    size = nodes.getSize()
    if weights.getDimension() != size:
        raise ValueError(
            "Nodes have size %d, but weights have dimension %d"
            % (size, weights.getDimension())
        )
    sortedNodes = nodes[indices]
    sortedWeights = weights[indices]
    return sortedNodes, sortedWeights


def testTensorProductExperiment1():
    # Generate a tensorized Gauss-Legendre rule in 2 dimensions.
    # The first marginal has 3 nodes.
    # The first marginal has 5 nodes.
    experiment1 = ot.GaussProductExperiment(ot.Uniform(0.0, 1.0), [3])
    experiment2 = ot.GaussProductExperiment(ot.Uniform(0.0, 1.0), [5])
    collection = [experiment1, experiment2]
    multivariateExperiment = ot.TensorProductExperiment(collection)
    nodes, weights = multivariateExperiment.generateWithWeights()
    # Sort
    nodes, weights = sortNodesAndWeights(nodes, weights)
    nodesExact = [
        [0.11270, 0.04691],
        [0.11270, 0.23076],
        [0.11270, 0.5],
        [0.11270, 0.76923],
        [0.11270, 0.95309],
        [0.5, 0.04691],
        [0.5, 0.23076],
        [0.5, 0.5],
        [0.5, 0.76923],
        [0.5, 0.95309],
        [0.88729, 0.04691],
        [0.88729, 0.23076],
        [0.88729, 0.5],
        [0.88729, 0.76923],
        [0.88729, 0.95309],
    ]
    weightsExact = [
        0.0329065,
        0.0664762,
        0.0790123,
        0.0664762,
        0.0329065,
        0.0526504,
        0.106362,
        0.12642,
        0.106362,
        0.0526504,
        0.0329065,
        0.0664762,
        0.0790123,
        0.0664762,
        0.0329065,
    ]
    rtol = 0.0
    atol = 1.0e-5
    ott.assert_almost_equal(nodes, nodesExact, rtol, atol)
    ott.assert_almost_equal(weights, weightsExact, rtol, atol)
    #
    size = multivariateExperiment.getSize()
    assert size == 15
    #
    distribution = multivariateExperiment.getDistribution()
    collection = [ot.Uniform(0.0, 1.0), ot.Uniform(0.0, 1.0)]
    expected_distribution = ot.BlockIndependentDistribution(collection)
    assert distribution == expected_distribution


def testTensorProductExperiment2():
    # Generate a tensorized Gauss-Legendre rule in 3 dimensions.
    # Each marginal elementary experiment has 6 nodes.
    dimension = 3
    maximumMarginalLevel = 6
    marginalLevels = [maximumMarginalLevel] * dimension
    collection = []
    for i in range(dimension):
        marginalExperiment = ot.GaussProductExperiment(
            ot.Uniform(0.0, 1.0), [marginalLevels[i]]
        )
        collection.append(marginalExperiment)
    multivariateExperiment = ot.TensorProductExperiment(collection)
    nodes, weights = multivariateExperiment.generateWithWeights()
    assert nodes.getDimension() == 3
    size = nodes.getSize()
    assert size == weights.getDimension()
    assert size == maximumMarginalLevel**dimension


def testTensorProductExperiment3():
    # Experiment 1 : Uniform * 2 with 3 and 2 nodes.
    marginalSizes1 = [3, 2]
    dimension1 = len(marginalSizes1)
    distribution1 = ot.JointDistribution([ot.Uniform()] * dimension1)
    experiment1 = ot.GaussProductExperiment(distribution1, marginalSizes1)
    # Experiment 2 : Normal * 3 with 2, 2 and 1 nodes.
    marginalSizes2 = [2, 2, 1]
    dimension2 = len(marginalSizes2)
    distribution2 = ot.JointDistribution([ot.Normal()] * dimension2)
    experiment2 = ot.GaussProductExperiment(distribution2, marginalSizes2)
    # Tensor product
    collection = [experiment1, experiment2]
    multivariateExperiment = ot.TensorProductExperiment(collection)
    nodes, weights = multivariateExperiment.generateWithWeights()
    # Sort
    nodes, weights = sortNodesAndWeights(nodes, weights)
    assert nodes.getDimension() == 5
    assert nodes.getSize() == 24
    assert weights.getSize() == 24

    nodesExact = [
        [
            -0.77459,
            -0.57735,
            -1.0,
            -1.0,
            0.0,
        ],
        [
            -0.77459,
            -0.57735,
            -1.0,
            1.0,
            0.0,
        ],
        [
            -0.77459,
            -0.57735,
            1.0,
            -1.0,
            0.0,
        ],
        [
            -0.77459,
            -0.57735,
            1.0,
            1.0,
            0.0,
        ],
        [
            -0.77459,
            0.57735,
            -1.0,
            -1.0,
            0.0,
        ],
        [
            -0.77459,
            0.57735,
            -1.0,
            1.0,
            0.0,
        ],
        [
            -0.77459,
            0.57735,
            1.0,
            -1.0,
            0.0,
        ],
        [
            -0.77459,
            0.57735,
            1.0,
            1.0,
            0.0,
        ],
        [
            0.0,
            -0.57735,
            -1.0,
            -1.0,
            0.0,
        ],
        [
            0.0,
            -0.57735,
            -1.0,
            1.0,
            0.0,
        ],
        [
            0.0,
            -0.57735,
            1.0,
            -1.0,
            0.0,
        ],
        [
            0.0,
            -0.57735,
            1.0,
            1.0,
            0.0,
        ],
        [
            0.0,
            0.57735,
            -1.0,
            -1.0,
            0.0,
        ],
        [
            0.0,
            0.57735,
            -1.0,
            1.0,
            0.0,
        ],
        [
            0.0,
            0.57735,
            1.0,
            -1.0,
            0.0,
        ],
        [
            0.0,
            0.57735,
            1.0,
            1.0,
            0.0,
        ],
        [
            0.77459,
            -0.57735,
            -1.0,
            -1.0,
            0.0,
        ],
        [
            0.77459,
            -0.57735,
            -1.0,
            1.0,
            0.0,
        ],
        [
            0.77459,
            -0.57735,
            1.0,
            -1.0,
            0.0,
        ],
        [
            0.77459,
            -0.57735,
            1.0,
            1.0,
            0.0,
        ],
        [
            0.77459,
            0.57735,
            -1.0,
            -1.0,
            0.0,
        ],
        [
            0.77459,
            0.57735,
            -1.0,
            1.0,
            0.0,
        ],
        [
            0.77459,
            0.57735,
            1.0,
            -1.0,
            0.0,
        ],
        [
            0.77459,
            0.57735,
            1.0,
            1.0,
            0.0,
        ],
    ]
    weightsExact = [
        0.0347222,
        0.0347222,
        0.0347222,
        0.0347222,
        0.0347222,
        0.0347222,
        0.0347222,
        0.0347222,
        0.0555556,
        0.0555556,
        0.0555556,
        0.0555556,
        0.0555556,
        0.0555556,
        0.0555556,
        0.0555556,
        0.0347222,
        0.0347222,
        0.0347222,
        0.0347222,
        0.0347222,
        0.0347222,
        0.0347222,
        0.0347222,
    ]
    rtol = 0.0
    atol = 1.0e-5
    ott.assert_almost_equal(nodes, nodesExact, rtol, atol)
    ott.assert_almost_equal(weights, weightsExact, rtol, atol)


# Testing
testTensorProductExperiment1()
testTensorProductExperiment2()
testTensorProductExperiment3()


# Check polynomial degree of exactness
def checkPolynomialExactness(
    marginalDegrees,
    marginalSizes,
    lowerBound=0.0,
    upperBound=1.0,
    rtol=1.0e-15,
    atol=0.0,
    verbose=False,
):
    """
    Check polynomial exactness of Gauss tensor product quadrature

    Parameters
    ----------
    marginalDegrees : list of int
        The polynomial degree of the marginal polynomials to integrate
    marginalSizes : list of int
        The number of nodes on each marginal axis.

    Examples
    --------
    marginalDegrees = [5, 3, 7]
    marginalSizes = [3, 2, 4]  # Polynomial exactness space = P5 x P3 x P7
    checkPolynomialExactness(marginalDegrees, marginalSizes)
    """
    dimension = len(marginalDegrees)
    if len(marginalSizes) != dimension:
        raise ValueError(
            f"Number of marginal degrees is {dimension} "
            f"but number of marginal sizes is {len(marginalSizes)}"
        )

    # Set bounds
    bounds = ot.Interval([lowerBound] * dimension, [upperBound] * dimension)

    # Create polynomial
    polynomialCollection = ot.PolynomialCollection()
    for i in range(dimension):
        coefficients = [0.0] * (1 + marginalDegrees[i])
        coefficients[-1] = 1
        polynomial = ot.UniVariatePolynomial(coefficients)
        polynomialCollection.add(polynomial)

    productPoly = ot.ProductPolynomialEvaluation(polynomialCollection)

    # Create Gauss tensor product quadrature
    lowerBound = bounds.getLowerBound()
    upperBound = bounds.getUpperBound()
    collection = [
        ot.GaussProductExperiment(ot.Uniform(lowerBound[i], upperBound[i]))
        for i in range(dimension)
    ]
    experiment = ot.TensorProductExperiment(collection)

    # Evaluate integral
    nodes, weights = experiment.generateWithWeights()
    values = productPoly(nodes).asPoint()
    computedIntegral = weights.dot(values)

    # Expected integral
    expectedIntegral = 1.0
    for i in range(dimension):
        marginalIntegral = (
            upperBound[i] ** (1 + marginalDegrees[i])
            - lowerBound[i] ** (1 + marginalDegrees[i])
        ) / (1 + marginalDegrees[i])
        expectedIntegral *= marginalIntegral
    absoluteError = abs(computedIntegral - expectedIntegral)
    relativeError = abs(computedIntegral - expectedIntegral) / abs(expectedIntegral)
    if verbose:
        print(
            f"Polynomial : {str(productPoly):20s}, "
            f"  integral computed  = {computedIntegral:.7f}, "
            f"  expected = {expectedIntegral:.7f}, "
            f"  abs. err. = {absoluteError:.3e}"
            f"  rel. err. = {relativeError:.3e}"
        )
    ott.assert_almost_equal(expectedIntegral, computedIntegral, rtol, atol)
    return


# Test different polynomials, up to the maximum
marginalSizes = [3, 2, 4]  # Polynomial exactness space = P5 x P3 x P7
maximumMarginalDegrees = [2 * n for n in marginalSizes]
experiment = ot.Tuples(maximumMarginalDegrees)
marginalDegreesList = experiment.generate()
for i in range(marginalDegreesList.getSize()):
    marginalDegrees = marginalDegreesList[i]
    checkPolynomialExactness(
        marginalDegrees, marginalSizes, rtol=1.0e-14, verbose=False
    )



"""
Use case : stiffened panel
==========================
"""

import openturns as ot


class StiffenedPanel:
    r"""
    Data class for the stiffened panel model.


    Attributes
    ----------

    dim : int
        The dimension of the problem, dim=10

    model : :class:`~openturns.SymbolicFunction`
        Model of the critical shearing load.
        The model has input dimension 10 and output dimension 1.
        More precisely, we have :math:`\vect{X} = (E, \nu, h_c, \ell, f_1, f_2, t, a, b_0, p)`
        and :math:`Y = (N_{xy})_{cr}`.

    E : :class:`~openturns.TruncatedNormal`
        Young modulus distribution (Pa),
        `ot.TruncatedNormal(110.0e9, 55.0e9, 99.0e9, 121.0e9)`

    nu : :class:`~openturns.Uniform`
        Poisson coefficient (-) distribution
        `ot.Uniform(0.3675, 0.3825)`

    h_c : :class:`~openturns.Uniform`
        Distance between the mean surface of the hat and the foot of the Stiffener (m) distribution
        `ot.Uniform(0.0285, 0.0315)`

    ell : :class:`~openturns.Uniform`
        Length of the stiffener flank (m) distribution
        `ot.Uniform(0.04655, 0.05145)`

    f_1 : :class:`~openturns.Uniform`
        Width of the stiffener foot (m) distribution
        `ot.Uniform(0.0266, 0.0294)`

    f_2 : :class:`~openturns.Uniform`
        Width of the stiffener hat (m) distribution
        `ot.Uniform(0.00627, 0.00693)`

    t : :class:`~openturns.Uniform`
        Thickness of the panel and the stiffener (m) distribution
        `ot.Uniform(8.02e-5, 8.181e-5)`

    a : :class:`~openturns.Uniform`
        Width of the panel (m)  distribution
        `ot.Uniform(0.6039, 0.6161)`

    b_0 : :class:`~openturns.Uniform`
        Distance between two stiffeners (m) distribution
        `ot.Uniform(0.04455, 0.04545)`

    p : :class:`~openturns.Uniform`
        Half-width of the stiffener (m) distribution
        `ot.Uniform(0.03762, 0.03838)`

    correlation_matrix : :class:`~openturns.CorrelationMatrix`
        The correlation matrix used for inputs dependence, mostly identity except for the term representing the interaction between variables :math:`f_1` and :math:`f_2` which is -0.8.

    copula : :class:`~openturns.NormalCopula`
        The (Normal) copula used to define the distribution of the input parameters.

    distribution : :class:`~openturns.JointDistribution`
        The joint distribution of the input parameters.

    independentDistribution : :class:`~openturns.JointDistribution`
        The joint distribution of the input parameters for the special case of independence.

    Examples
    --------
    >>> from openturns.usecases import stiffened_panel
    >>> # Load the stiffened panel model
    >>> panel = stiffened_panel.StiffenedPanel()
    >>> print("Inputs:", panel.model.getInputDescription())
    Inputs: [E,nu,h_c,ell,f_1,f_2,t,a,b_0,p]
    >>> print("Outputs:", panel.model.getOutputDescription())
    Outputs: [(N_{xy})_{cr}]
    """

    def __init__(self):
        self.dim = 10

        # Here we use substitutions in order to get a unique formula for the model.
        # This way, we get a symbolic derivation of the gradient and the hessian,
        # which is not possible if we use the exprtk embedded language
        formula = "(5.35 + 4 * (b_0 / a)^2) * (pi_^2 / b_0^2 * E * t^3 / (12 * (1 - nu^2))) * (1 + (2 * p * (h - 2 * h_0) - h_c  * (f_1 - f_2)) / (4 * h_0 * ell))"
        # Substitutions
        a = "(ell*t)"
        a_bar = "(A+t*(p+(f_1-f_2)/2))"
        h_0 = "((A*(h_c+2*t)+t^2*(f_1-f_2))/(2*A_bar))"
        h = "(h_c + t)"
        formula = formula.replace("h_0", h_0)
        formula = formula.replace("A_bar", a_bar)
        formula = formula.replace("A", a)
        # Attention! If the substitution of h by its formula is done without
        # caution h_c will be replaced by (h_c + t)_c!
        # First, we protect h_c
        formula = formula.replace("h_c", "H_C")
        # Then we replace h by its formula
        formula = formula.replace("h", h)
        # On restaure les occurrences de h_c
        formula = formula.replace("H_C", "h_c")
        self.model = ot.SymbolicFunction(
            ["E", "nu", "h_c", "ell", "f_1", "f_2", "t", "a", "b_0", "p"], [formula]
        )
        self.model.setOutputDescription([r"(N_{xy})_{cr}"])

        self.youngModulus = ot.TruncatedNormal(110.0e9, 55.0e9, 99.0e9, 121.0e9)
        self.youngModulus.setDescription(["E (Pa)"])

        self.nu = ot.Uniform(0.3675, 0.3825)
        self.nu.setDescription(["nu (-)"])

        self.h_c = ot.Uniform(0.0285, 0.0315)
        self.h_c.setDescription(["h_c (m)"])

        self.ell = ot.Uniform(0.04655, 0.05145)
        self.ell.setDescription(["ell (m)"])

        self.f_1 = ot.Uniform(0.0266, 0.0294)
        self.f_1.setDescription(["f_1 (m)"])

        self.f_2 = ot.Uniform(0.00627, 0.00693)
        self.f_2.setDescription(["f_2 (m)"])

        self.t = ot.Uniform(8.02e-5, 8.181e-5)
        self.t.setDescription(["t (m)"])

        self.a = ot.Uniform(0.6039, 0.6161)
        self.a.setDescription(["a (m)"])

        self.b_0 = ot.Uniform(0.04455, 0.04545)
        self.b_0.setDescription(["b_0 (m)"])

        self.p = ot.Uniform(0.03762, 0.03838)
        self.p.setDescription(["p (m)"])

        # correlation matrix
        self.correlation_matrix = ot.CorrelationMatrix(self.dim)
        self.correlation_matrix[4, 5] = -0.8
        self.copula = ot.NormalCopula(
            ot.NormalCopula.GetCorrelationFromSpearmanCorrelation(
                self.correlation_matrix
            )
        )

        varList = [
            self.youngModulus,
            self.nu,
            self.h_c,
            self.ell,
            self.f_1,
            self.f_2,
            self.t,
            self.a,
            self.b_0,
            self.p,
        ]

        # distribution
        self.distribution = ot.JointDistribution(varList, self.copula)

        # special case: independent distribution
        self.independentDistribution = ot.JointDistribution(varList)



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()

# Instantiate one distribution object
distribution = ot.VonMises(-0.5, 1.5)
print("Distribution ", distribution)

# Is this distribution elliptical ?
print("Elliptical = ", distribution.isElliptical())

# Is this distribution continuous ?
print("Continuous = ", distribution.isContinuous())

# Test for realization of distribution
oneRealization = distribution.getRealization()
print("oneRealization=", oneRealization)

# Define a point
point = ot.Point(distribution.getDimension(), 1.0)
print("Point= ", point)

# Show PDF and CDF of point
eps = 1e-5
DDF = distribution.computeDDF(point)
print("ddf     =", DDF)
LPDF = distribution.computeLogPDF(point)
print("log pdf= %.12g" % LPDF)
PDF = distribution.computePDF(point)
print("pdf     =%.6f" % PDF)

CDF = distribution.computeCDF(point)
print("cdf= %.12g" % CDF)
CCDF = distribution.computeComplementaryCDF(point)
print("ccdf= %.12g" % CCDF)
Survival = distribution.computeSurvivalFunction(point)
print("survival= %.12g" % Survival)
CF = distribution.computeCharacteristicFunction(point[0])
print("characteristic function=(%.6g, %.6g)" % (CF.real, CF.imag))
LCF = distribution.computeLogCharacteristicFunction(point[0])
print("log characteristic function=(%.6g, %.6g)" % (LCF.real, LCF.imag))
PDFgr = distribution.computePDFGradient(point)
print("pdf gradient     =", PDFgr)

CDFgr = distribution.computeCDFGradient(point)
print("cdf gradient     =", CDFgr)

quantile = distribution.computeQuantile(0.95)
print("quantile=", quantile)
print("cdf(quantile)= %.12g" % distribution.computeCDF(quantile))
# Get 95% survival function
inverseSurvival = ot.Point(distribution.computeInverseSurvivalFunction(0.95))
print("InverseSurvival=", repr(inverseSurvival))
print(
    "Survival(inverseSurvival)=%.6f"
    % distribution.computeSurvivalFunction(inverseSurvival)
)
print("entropy=%.6f" % distribution.computeEntropy())

# Confidence regions
interval, threshold = distribution.computeMinimumVolumeIntervalWithMarginalProbability(
    0.95
)
print("Minimum volume interval=", interval)
print("threshold=", ot.Point(1, threshold))
levelSet, beta = distribution.computeMinimumVolumeLevelSetWithThreshold(0.95)
print("Minimum volume level set=", levelSet)
print("beta=", ot.Point(1, beta))
interval, beta = distribution.computeBilateralConfidenceIntervalWithMarginalProbability(
    0.95
)
print("Bilateral confidence interval=", interval)
print("beta=", ot.Point(1, beta))
(
    interval,
    beta,
) = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(0.95, False)
print("Unilateral confidence interval (lower tail)=", interval)
print("beta=", ot.Point(1, beta))
(
    interval,
    beta,
) = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(0.95, True)
print("Unilateral confidence interval (upper tail)=", interval)
print("beta=", ot.Point(1, beta))

mean = distribution.getMean()
print("mean=", mean)
standardDeviation = distribution.getStandardDeviation()
print("standard deviation=", standardDeviation)
skewness = distribution.getSkewness()
print("skewness=", skewness)
kurtosis = distribution.getKurtosis()
print("kurtosis=", kurtosis)
covariance = distribution.getCovariance()
print("covariance=", covariance)
correlation = distribution.getCorrelation()
print("correlation=", correlation)
spearman = distribution.getSpearmanCorrelation()
print("spearman=", spearman)
kendall = distribution.getKendallTau()
print("kendall=", kendall)
parameters = distribution.getParametersCollection()
print("parameters=", parameters)
print("Standard representative=", distribution.getStandardRepresentative())
print("Circular mean=", distribution.getCircularMean())
print("Circular variance= %.12g" % distribution.getCircularVariance())

ot.Log.Show(ot.Log.TRACE)
validation = ott.DistributionValidation(distribution)
validation.run()



#! /usr/bin/env python

import openturns as ot
from openturns.testing import assert_almost_equal

ot.TESTPREAMBLE()


# TEST NUMBER ZERO : DEFAULT CONSTRUCTOR AND STRING CONVERTER */
print("test number zero : default constructor and string converter")

# Default constructor */
squareMatrix0 = ot.SquareMatrix()

# String converter */
print("squareMatrix0 = ", repr(squareMatrix0))

# TEST NUMBER ONE : CONSTRUCTOR WITH SIZE, OPERATOR() AND STRING CONVERTER
# */
print("test number one : constructor with size, operator() and string converter")

# Constructor with size */
squareMatrix1 = ot.SquareMatrix(2)

# Check operator() methods */
squareMatrix1[0, 0] = 1.0
squareMatrix1[1, 0] = 2.0
squareMatrix1[0, 1] = 3.0
squareMatrix1[1, 1] = 4.0

# String converter */
print("squareMatrix1 = ", repr(squareMatrix1))

# TEST NUMBER TWO : COPY CONSTRUCTOR AND STRING CONVERTER */
print("test number two : copy constructor and string converter")

# Copy constructor */
squareMatrix2 = ot.SquareMatrix(squareMatrix1)

# String converter */
print("squareMatrix2 = ", repr(squareMatrix2))

# TEST NUMBER THREE : GET DIMENSIONS METHODS */
print("test number three : get dimensions methods")

# Get dimension methods */
print("squareMatrix1's nbRows = ", squareMatrix1.getNbRows())
print("squareMatrix1's nbColumns = ", squareMatrix1.getNbColumns())

# TEST NUMBER FIVE : ASSIGNMENT METHOD */
print("test number five : assignment method")

# Assignment method */
# non sense with python

# TEST NUMBER SIX : TRANSPOSITION METHOD */
print("test number six : transposition method")

# Check transpose method */
squareMatrix4 = squareMatrix1.transpose()
print("squareMatrix1 transposed = ", repr(squareMatrix4))
print("squareMatrix1 transposed = ", repr(squareMatrix1.transpose()))

# TEST NUMBER SEVEN : ADDITION METHOD */
print("test number seven : addition method")

# Check addition method : we check the operator and the symmetry of the
# operator, thus testing the comparison operator */
sum1 = squareMatrix1 + squareMatrix4
sum2 = squareMatrix4 + squareMatrix1
print("sum1 = ", repr(sum1))
print("sum2 = ", repr(sum2))
print("sum1 equals sum2 = ", (sum1 == sum2))

# TEST NUMBER EIGHT : SUBTRACTION METHOD */
print("test number eight : subtraction method")

# Check subtraction method */
diff = squareMatrix1 - squareMatrix4
print("diff = ", repr(diff))

# TEST NUMBER NINE : MATRIX MULTIPLICATION METHOD */
print("test number nine : matrix multiplication method")

# Check multiplication method */
prod = squareMatrix1 * squareMatrix4
print("prod = ", repr(prod))

# TEST NUMBER TEN : MULTIPLICATION WITH A NUMERICAL POINT METHOD */
print("test number ten : multiplication with a numerical point method")

# Create the numerical point */
pt = ot.Point()
pt.add(1.0)
pt.add(2.0)
print("pt = ", repr(pt))

# Check the product method */
ptResult = squareMatrix1 * pt
print("ptResult = ", repr(ptResult))

# TEST NUMBER ELEVEN : MULTIPLICATION AND DIVISION BY A NUMERICAL SCALAR
# METHODS */
print("test number eleven : multiplication and division by a numerical scalar methods")

# Check the multiplication method */
s = 3.0
scalprod1 = squareMatrix1 * s
# bug PYTHON scalprod2 = s * matrix1
scalprod2 = squareMatrix1 * s
print("scalprod1 = ", repr(scalprod1))
print("scalprod2 = ", repr(scalprod2))
print("scalprod1 equals scalprod2 = ", (scalprod1 == scalprod2))

# Check the division method */
scaldiv1 = squareMatrix1 / s
scaldiv2 = squareMatrix1 / s
print("scaldiv1 = ", repr(scaldiv1))
print("scaldiv2 = ", repr(scaldiv2))
print("scaldiv1 equals scaldiv2 = ", (scaldiv1 == scaldiv2))

# TEST NUMBER TWELVE : ISEMPTY METHOD */
print("test number twelve : isEmpty method")

# Check method isEmpty */
squareMatrix5 = ot.SquareMatrix()
print("squareMatrix0 is empty = ", squareMatrix0.isEmpty())
print("squareMatrix1 is empty = ", squareMatrix1.isEmpty())
print("squareMatrix5 is empty = ", squareMatrix5.isEmpty())

# Check inverse()
squareMatrix6 = ot.SquareMatrix([[1.0, 2.0, 3.0], [3.0, 2.0, 1.0], [2.0, 1.0, 3.0]])
squareMatrix7 = squareMatrix6.inverse()
inverseReference = ot.SquareMatrix(
    [[-5.0, 3.0, 4.0], [7.0, 3.0, -8.0], [1.0, -3.0, 4.0]]
)
inverseReference /= 12.0
assert_almost_equal(squareMatrix7, inverseReference)



"""
Create an extreme value distribution
====================================
"""

# %%
# Abstract
# --------
#
# In this example we show how to define extreme values distributions.

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# The generalized extreme value distribution (GEV)
# ------------------------------------------------

# %%
# The :class:`~openturns.GeneralizedExtremeValue` distribution is a family of continuous probability distributions
# that combines the :class:`~openturns.Gumbel`, :class:`~openturns.Frechet` and :class:`~openturns.WeibullMax` distributions,
# all extreme value distributions.
#
# We define a generalized extreme value distribution with parameters :math:`\mu = 0.0`, :math:`\sigma = 1.0` and :math:`\xi = 0.0`
myDistribution = ot.GeneralizedExtremeValue(0.0, 1.0, 0.0)

# %%
# The :class:`~openturns.GeneralizedExtremeValue` class acts as a proxy class. We can get the actual distribution (Weibull, Frechet or Gumbel) with the `getActualDistribution` method:
print(myDistribution.getActualDistribution())

# %%
# For the chosen parameters it is a Gumbel distribution with parameters
# :math:`\beta=1` and :math:`\gamma = 0`.

# %%
# We draw its PDF and CDF :
graphPDF = myDistribution.drawPDF()
graphPDF.setTitle(
    r"PDF of the GEV with parameters $\mu = 0.0$, $\sigma = 1.0$ and $\xi = 0.0$ "
)
view = otv.View(graphPDF)

graphCDF = myDistribution.drawCDF()
graphCDF.setTitle(
    r"CDF of the GEV with parameters $\mu = 0.0$, $\sigma = 1.0$ and $\xi = 0.0$ "
)
view = otv.View(graphCDF)

# %%
# The Generalized Pareto Distribution (GPD)
# -----------------------------------------
#
# In this paragraph we turn to the definition of a :class:`~openturns.GeneralizedPareto` distribution.
# For instance we build a generalized Pareto distribution with parameters :math:`\sigma = 1.0`, :math:`\xi = 0.0` and :math:`u = 0.0` :
#
myGPD = ot.GeneralizedPareto(1.0, 0.0, 0.0)

# %%
# We draw its PDF and CDF :
graphPDF = myGPD.drawPDF()
graphPDF.setTitle(
    r"PDF of the GPD with parameters $\sigma = 1.0$, $\xi = 0.0$ and $u = 0.0$ "
)
view = otv.View(graphPDF)

# %%
graphCDF = myGPD.drawCDF()
graphCDF.setTitle(
    r"CDF of the GPD with parameters $\sigma = 1.0$, $\xi = 0.0$ and $u = 0.0$ "
)
view = otv.View(graphCDF)

# %%
# Display all figures
otv.View.ShowAll()



"""
Aggregate processes
===================
"""

# %%
# In this example we are going to concatenate several processes that share the same mesh.

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# Create processes to aggregate
myMesher = ot.IntervalMesher([100, 10])
lowerbound = [0.0, 0.0]
upperBound = [2.0, 4.0]
myInterval = ot.Interval(lowerbound, upperBound)
myMesh = myMesher.build(myInterval)
myProcess1 = ot.WhiteNoise(ot.Normal(), myMesh)
myProcess2 = ot.WhiteNoise(ot.Triangular(), myMesh)


# %%
# Draw values of a realization of the 2nd process
marginal = ot.HistogramFactory().build(myProcess1.getRealization().getValues())
graph = marginal.drawPDF()
view = otv.View(graph)

# %%
# Create an aggregated process
myAggregatedProcess = ot.AggregatedProcess([myProcess1, myProcess2])

# %%
# Draw values of the realization on the 2nd marginal
marginal = ot.HistogramFactory().build(
    myAggregatedProcess.getRealization().getValues().getMarginal(0)
)
graph = marginal.drawPDF()
_ = otv.View(graph)

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()


ot.ResourceMap.SetAsUnsignedInteger("MeixnerDistribution-CDFDiscretization", 100)
ot.ResourceMap.SetAsScalar("MeixnerDistribution-MaximumAbsoluteError", 1.0e-6)
ot.ResourceMap.SetAsScalar("MeixnerDistribution-MaximumRelativeError", 1.0e-6)
ot.ResourceMap.SetAsScalar("MeixnerDistribution-MaximumConstraintError", 1.0e-6)
ot.ResourceMap.SetAsScalar("MeixnerDistribution-MaximumObjectiveError", 1.0e-6)

# Instantiate one distribution object
distribution = ot.MeixnerDistribution(1.5, 0.5, 2.5, -0.5)
print("Distribution ", repr(distribution))
print("Distribution ", distribution)

# Is this distribution elliptical ?
print("Elliptical = ", distribution.isElliptical())

# Is this distribution continuous ?
print("Continuous = ", distribution.isContinuous())

# Test for realization of distribution
oneRealization = distribution.getRealization()
print("oneRealization=", oneRealization)

# Define a point
point = ot.Point(distribution.getDimension(), 1.0)
print("Point= ", point)

# Show PDF and CDF of point
LPDF = distribution.computeLogPDF(point)
print("log pdf=%.6f" % LPDF)
PDF = distribution.computePDF(point)
print("pdf     =%.6f" % PDF)

print("PDF gradient=", distribution.computePDFGradient(point))
CDF = distribution.computeCDF(point)
print("cdf=%.6f" % CDF)
print("CDF gradient=", distribution.computeCDFGradient(point))
CCDF = distribution.computeComplementaryCDF(point)
print("ccdf=%.6f" % CCDF)
CF = distribution.computeCharacteristicFunction(point[0])
print("characteristic function=(%.12g%+.12gj)" % (CF.real, CF.imag))
LCF = distribution.computeLogCharacteristicFunction(point[0])
print("log characteristic function=(%.12g%+.12gj)" % (LCF.real, LCF.imag))
quantile = distribution.computeQuantile(0.95)
print("quantile=", quantile)
print("cdf(quantile)=%.6f" % distribution.computeCDF(quantile))
# Get 95% survival function
inverseSurvival = ot.Point(distribution.computeInverseSurvivalFunction(0.95))
print("InverseSurvival=", repr(inverseSurvival))
print(
    "Survival(inverseSurvival)=%.6f"
    % distribution.computeSurvivalFunction(inverseSurvival)
)
print("entropy=%.6f" % distribution.computeEntropy())

# Confidence regions
interval, threshold = distribution.computeMinimumVolumeIntervalWithMarginalProbability(
    0.95
)
print("Minimum volume interval=", interval)
print("threshold=", ot.Point(1, threshold))
levelSet, beta = distribution.computeMinimumVolumeLevelSetWithThreshold(0.95)
print("Minimum volume level set=", levelSet)
print("beta=", ot.Point(1, beta))
interval, beta = distribution.computeBilateralConfidenceIntervalWithMarginalProbability(
    0.95
)
print("Bilateral confidence interval=", interval)
print("beta=", ot.Point(1, beta))
(
    interval,
    beta,
) = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(0.95, False)
print("Unilateral confidence interval (lower tail)=", interval)
print("beta=", ot.Point(1, beta))
(
    interval,
    beta,
) = distribution.computeUnilateralConfidenceIntervalWithMarginalProbability(0.95, True)
print("Unilateral confidence interval (upper tail)=", interval)
print("beta=", ot.Point(1, beta))

mean = distribution.getMean()
print("mean=", mean)
covariance = distribution.getCovariance()
print("covariance=", covariance)
parameters = distribution.getParametersCollection()
print("parameters=", parameters)
print("Standard representative=", distribution.getStandardRepresentative())

# Specific to this distribution
alpha = distribution.getAlpha()
print("alpha=%.6f" % alpha)
beta = distribution.getBeta()
print("beta=%.6f" % beta)
delta = distribution.getDelta()
print("delta=%.6f" % delta)
gamma = distribution.getGamma()
print("gamma=%.6f" % gamma)
standardDeviation = distribution.getStandardDeviation()
print("standard deviation=", standardDeviation)
skewness = distribution.getSkewness()
print("skewness=", skewness)
kurtosis = distribution.getKurtosis()
print("kurtosis=", kurtosis)

ot.Log.Show(ot.Log.TRACE)
validation = ott.DistributionValidation(distribution)
validation.run()



#! /usr/bin/env python

import openturns as ot
from openturns.testing import assert_almost_equal


f = ot.SymbolicFunction("x", "x^2")
f = ot.MemoizeFunction(f)
f.disableHistory()
print(f)
size = 4
inputSample = ot.Sample(size, 1)
for i in range(size):
    inputSample[i, 0] = i
outputSample = f(inputSample)
print("Is history enabled for f? ", f.isHistoryEnabled())
print("input history=", f.getInputHistory())
print("output history=", f.getOutputHistory())
f.enableHistory()
outputSample = f(inputSample)
print("Is history enabled for f? ", f.isHistoryEnabled())
print("input history=", f.getInputHistory())
print("output history=", f.getOutputHistory())
f.clearHistory()
print("Is history enabled for f? ", f.isHistoryEnabled())
print("input history=", f.getInputHistory())
print("output history=", f.getOutputHistory())
# Perform the computation twice
outputSample = f(inputSample)
outputSample = f(inputSample)
print("input history=", f.getInputHistory())
print("output history=", f.getOutputHistory())

mem = ot.MemoizeFunction(ot.SymbolicFunction(["x1", "x2"], ["x1+x2", "3*(x1+x2)"]))
mem([1, 2])
mem2 = ot.MemoizeFunction(mem.getMarginal(1))
mem2([1, 2])
print("mem2.in", mem2.getCacheInput())
print("mem2.out", mem2.getCacheOutput())
print("mem2.hits", mem2.getCacheHits())

# check getCacheInput/getCacheOutput order
for i in range(5):
    mem([i + 2, i + 3])
assert mem(mem.getCacheInput()) == mem.getCacheOutput(), "wrong order"
print("ok")

ot.PlatformInfo.SetNumericalPrecision(20)


def py_f(X):
    return X


# Check if the meoization propagates through the finite difference gradients
# Here we use a PythonFunction as its gradient/hessian are based on finite
# differences by default
ot_f = ot.MemoizeFunction(ot.PythonFunction(3, 3, py_f))
x = [1.0, 2.0, 3.0]
n_calls_0 = ot_f.getEvaluationCallsNumber()
res_f = ot_f(x)
res_grad = ot_f.gradient(x)
res_hess = ot_f.hessian(x)
n_calls_1 = ot_f.getEvaluationCallsNumber()
# 25=1+6+18
assert_almost_equal(n_calls_1 - n_calls_0, 25, 0.0, 0.0)
# Do the computation once again
n_calls_0 = n_calls_1
res_f = ot_f(x)
res_grad = ot_f.gradient(x)
res_hess = ot_f.hessian(x)
n_calls_1 = ot_f.getEvaluationCallsNumber()
# 0=everything is reused
assert_almost_equal(n_calls_1 - n_calls_0, 0, 0.0, 0.0)
# Now, switch to noncentered gradients to reduce the calls to the minimum
eps = 1e-8
gr_f = ot.NonCenteredFiniteDifferenceGradient(eps, ot_f.getEvaluation())
ot_f.setGradient(gr_f)
x = [3, 1, 2]
n_calls_0 = n_calls_1
res_f = ot_f(x)
res_grad = ot_f.gradient(x)
res_hess = ot_f.hessian(x)
n_calls_1 = ot_f.getEvaluationCallsNumber()
# 22=1+3(3+1 reused)+18
assert_almost_equal(n_calls_1 - n_calls_0, 22, 0.0, 0.0)

# check that marginals share the same cache


def f_py(x):
    f_py.n += 1
    x0, x1 = x
    return [x0 + x1, x0 - x1]


f_py.n = 0
f_test = ot.MemoizeFunction(ot.PythonFunction(2, 2, f_py))
f1 = f_test.getMarginal(0)
f2 = f_test.getMarginal(1)
x = [1, 2]
f1(x)
f2(x)
assert f_py.n == 1, "only one eval"

# here we want f modified by ref though evals on f0
f = ot.MemoizeFunction(ot.SymbolicFunction(["x1", "x2"], ["x1+x2", "x1*x2"]))
x = ot.Normal(2).getSample(10)
f0 = ot.MemoizeFunction(f.getMarginal([0]))
f0(x)
assert f.getInputHistory().getSize() == len(x), "wrong size"
assert f.getCacheInput().getSize() == len(x), "wrong size"



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()
ot.PlatformInfo.SetNumericalPrecision(2)

m = 10
x = [[0.5 + i] for i in range(m)]

inVars = ["a", "b", "c", "x"]
formulas = ["a + b * exp(c * x)", "(a * x^2 + b) / (c + x^2)"]
g = ot.SymbolicFunction(inVars, formulas)
trueParameter = [2.8, 1.2, 0.5]
params = [0, 1, 2]
model = ot.ParametricFunction(g, params, trueParameter)
y = model(x)
y += ot.Normal([0.0] * 2, [0.05] * 2, ot.IdentityMatrix(2)).getSample(m)
candidate = [1.0] * 3
bootstrapSizes = [0, 100]
for bootstrapSize in bootstrapSizes:
    algo = ot.NonLinearLeastSquaresCalibration(model, x, y, candidate)
    algo.setBootstrapSize(bootstrapSize)
    algo.run()
    result = algo.getResult()
    # To avoid discrepance between the platforms with or without CMinpack
    print("result (Auto)=", result.getParameterMAP())
    ott.assert_almost_equal(
        result.getObservationsError().getMean(), [0.0051, -0.0028], 1e-1, 1e-3
    )
    multiStartSize = 10
    algo.setOptimizationAlgorithm(
        ot.MultiStart(
            ot.TNC(),
            ot.LowDiscrepancyExperiment(
                ot.SobolSequence(),
                ot.Normal(
                    candidate, ot.CovarianceMatrix(ot.Point(candidate).getDimension())
                ),
                multiStartSize,
            ).generate(),
        )
    )
    algo.run()
    result = algo.getResult()
    # To avoid discrepance between the platforms with or without CMinpack
    print("result  (TNC)=", result.getParameterMAP())
    ott.assert_almost_equal(
        result.getObservationsError().getMean(), [0.0051, -0.0028], 1e-1, 1e-3
    )
    # Draw result
    graph = result.drawParameterDistributions()
    graph = result.drawResiduals()
    graph = result.drawObservationsVsInputs()
    graph = result.drawObservationsVsPredictions()



"""
Combinatorial generators
========================
"""

# %%
# In this example we are going to expose the various design of experiments that allow one to generate all the integer collections satisfying a given combinatorial constraint:
#
# - The Tuples generator, which allows one to generate all the elements of a Cartesian product :math:`E=\{0,\dots,n_0-1\}\times\dots\times\{0,\dots,n_{d-1}-1\}`.
#
#   The total number of generated points is :math:`N=\prod_{k=0}^{d-1}n_k`.
#
# - The K-permutations generator, which allows one to generate all the injective functions from :math:`\{0,\dots,k-1\}` into :math:`\{0,\dots,n-1\}`
#
#   The total number of generated points is :math:`N=\dfrac{n!}{(n-k)!}`.
#
# - The Combinations generator, which allows one to generate all the subsets of size :math:`k` of :math:`\{0,\dots,n-1\}`
#
#   The total number of generated points is :math:`N=\dfrac{n!}{k!(n-k)!}`.
#

# %%
import openturns as ot


# %%
# Tuples
# ------
experiment = ot.Tuples([2, 3, 5])
print(experiment.generate())

# %%
# K-permutations
# --------------
experiment = ot.KPermutations(3, 4)
print(experiment.generate())

# %%
# Combinations
# ------------
experiment = ot.Combinations(4, 6)
print(experiment.generate())



import sympy as sp

x, a, b, a_, b_, point, gamma_, expX, beta_, alpha_, mu, sigma = sp.symbols(
    "x a b a_ b_ point gamma_ expX beta_ alpha_ mu sigma"
)

alpha = ((mu - a) / (b - a)) * (((b - mu) * (mu - a)) / (sigma * sigma) - 1)
print("dalpha/dmu", sp.simplify(sp.diff(alpha, mu)))
print("dalpha/dsigma", sp.simplify(sp.diff(alpha, sigma)))
print("dalpha/da", sp.simplify(sp.diff(alpha, a)))
print("dalpha/db", sp.simplify(sp.diff(alpha, b)))

beta = (((b - mu) * (mu - a)) / (sigma * sigma) - 1) * ((b - mu) / (b - a))
print("dbeta/dmu", sp.simplify(sp.diff(beta, mu)))
print("dbeta/dsigma", sp.simplify(sp.diff(beta, sigma)))
print("dbeta/da", sp.simplify(sp.diff(beta, a)))
print("dbeta/db", sp.simplify(sp.diff(beta, b)))



r"""
Create a Joint by Conditioning  distribution
============================================
"""

# %%
# In this example we are going to build the distribution of the random vector:
#
# .. math::
#    (Y, \vect{X}|\vect{\Theta})
#
# with :math:`\vect{X}` conditioned by the random vector :math:`\vect{\Theta}` obtained with the random variable :math:`Y` through a function :math:`f`:
#
# .. math::
#    \vect{\Theta} = f(\vect{Y})
#

# %%
import openturns as ot
import openturns.viewer as otv


# %%
# We consider the following case: :math:`X|\vect{\Theta} \sim \cN(\vect{\Theta})`
# with :math:`\vect{\Theta} = (Y, 0.1 + Y^2)` and :math:`Y \sim \cN(0,1)`.
#
# We first create the :math:`Y` distribution:
YDist = ot.Normal(0.0, 1.0)

# %%
# Then we create the link function :math:`f: y \rightarrow (y, 0.1+y^2)`:
f = ot.SymbolicFunction(["y"], ["y", "0.1 + y^2"])

# %%
# Then, we create the :math:`\vect{X}|\vect{\Theta}` distribution:
XgivenThetaDist = ot.Normal()

# %%
# At last, we create the distribution of :math:`(Y,X)`:
XDist = ot.JointByConditioningDistribution(XgivenThetaDist, YDist, f)
XDist.setDescription(["Y", r"$X|\mathbf{\Theta} = f(Y)$"])
XDist

# %%
# Get a sample:
sample = XDist.getSample(100)

# %%
# Draw the PDF:
ot.ResourceMap.SetAsString("Contour-DefaultColorMapNorm", "rank")
graph = XDist.drawPDF(sample.getMin(), sample.getMax(), [256] * 2)
graph.setTitle(r"$(Y,X)$ iso-PDF")
cloud = ot.Cloud(sample)
cloud.setColor("red")
cloud.setLegend("sample")
graph.add(cloud)
view = otv.View(graph)

# %%
view.ShowAll()



#! /usr/bin/env python

import openturns as ot

eps = 1e-2
# Instance creation
myFunc = ot.SymbolicFunction(
    ["x1", "x2"], ["x1*sin(x2)", "cos(x1+x2)", "(x2+1)*exp(x1-2*x2)"]
)
epsilon = ot.Point(myFunc.getInputDimension(), eps)
inPoint = ot.Point(epsilon.getDimension(), 1.0)
myHessian = ot.CenteredFiniteDifferenceHessian(epsilon, myFunc.getEvaluation())

print("myHessian=", repr(myHessian))
print("myFunc.hessian(", repr(inPoint), ")=", repr(myFunc.hessian(inPoint)))
print("myHessian.hessian(", repr(inPoint), ")=", repr(myHessian.hessian(inPoint)))

# Substitute the hessian
myFunc.setHessian(ot.CenteredFiniteDifferenceHessian(myHessian))
print(
    "myFunc.hessian(",
    repr(inPoint),
    ")=",
    repr(myFunc.hessian(inPoint)),
    " (after substitution)",
)



"""
We compare openturns models with R models in case of intercept/no intercept
we use small tests (5 exactly) mixing various situations
"""

import numpy as np
from rpy2.robjects import Formula, r

try:
    from rpy2.robjects.numpy2ri import numpy2ri, ri2py
except ImportError:
    from rpy2.robjects.numpy2ri import numpy2rpy as numpy2ri
    from rpy2.robjects.numpy2ri import rpy2py as ri2py

# from rpy2.robjects.numpy2ri import numpy2ri, ri2py
from rpy2.robjects.packages import importr, data

import openturns as ot

# Require

stats = importr("stats")
faraway = importr("faraway")
savings_data = data(faraway).fetch("savings")["savings"]
# data(faraway).fetch("savings")["savings"]

# Model 1 : 2 param, non intercept
sr = ri2py(savings_data)["sr"]
r.assign("sr", numpy2ri(sr))
pop15 = ri2py(savings_data)["pop15"]
r.assign("pop15", numpy2ri(pop15))
pop75 = ri2py(savings_data)["pop75"]
r.assign("pop75", numpy2ri(pop75))

formula = Formula("sr ~ pop75 + pop15 - 1")
fit = stats.lm(formula)
summary = stats.summary_lm(fit)

"""
list(summary.names) provides
['call', 'terms', 'residuals', 'coefficients', 'aliased', 'sigma', 'df', 'r.squared', 'adj.r.squared', 'fstatistic', 'cov.unscaled']
 """
r2 = summary[7][0]
ar2 = summary[8][0]
ftest = summary[9][0]

# openturns like
X = ot.Sample(len(pop15), 0)
P = ot.Sample.BuildFromPoint(pop15)
X.stack(P)
P = ot.Sample.BuildFromPoint(pop75)
X.stack(P)
X.setDescription(["pop15", "pop75"])
Y = ot.Sample.BuildFromPoint(sr)
# Model with pop15 + pop75 without intercept
# We do not run mode with pop15 only for the moment
f = ot.SymbolicFunction(["pop15", "pop75"], ["pop15"])
g = ot.SymbolicFunction(["pop15", "pop75"], ["pop75"])
h = ot.SymbolicFunction(["pop15", "pop75"], ["1"])
basis = ot.Basis([f, g])
model = ot.LinearModelAlgorithm(X, basis, Y)
model.run()
result = model.getResult()
analysis = ot.LinearModelAnalysis(result)

np.testing.assert_almost_equal(result.getRSquared(), r2, 14)
np.testing.assert_almost_equal(result.getAdjustedRSquared(), ar2, 14)
np.testing.assert_almost_equal(analysis.getFisherScore(), ftest, 14)

# ---------------------------------------------------------------------
# Model 2 : 2 param + intercepts

formula = Formula("sr ~ pop75 + pop15")
fit = stats.lm(formula)
summary = stats.summary_lm(fit)

r2 = summary[7][0]
ar2 = summary[8][0]
ftest = summary[9][0]

# openturns like
model = ot.LinearModelAlgorithm(X, Y)
model.run()
result = model.getResult()
analysis = ot.LinearModelAnalysis(result)

np.testing.assert_almost_equal(result.getRSquared(), r2, 14)
np.testing.assert_almost_equal(result.getAdjustedRSquared(), ar2, 14)
np.testing.assert_almost_equal(analysis.getFisherScore(), ftest, 14)

# ---------------------------------------------------------------------
# Model 3 : 1 param (pop15) + intercepts
formula = Formula("sr ~ pop15")
fit = stats.lm(formula)
summary = stats.summary_lm(fit)

r2 = summary[7][0]
ar2 = summary[8][0]
ftest = summary[9][0]

# openturns like
basis = ot.Basis([f, h])
model = ot.LinearModelAlgorithm(X, basis, Y)
model.run()
result = model.getResult()
analysis = ot.LinearModelAnalysis(result)

np.testing.assert_almost_equal(result.getRSquared(), r2, 14)
np.testing.assert_almost_equal(result.getAdjustedRSquared(), ar2, 14)
np.testing.assert_almost_equal(analysis.getFisherScore(), ftest, 14)

# ---------------------------------------------------------------------
# Model 4 : 1 param (pop15) + no intercept intercepts
formula = Formula("sr ~ pop15 - 1")
fit = stats.lm(formula)
summary = stats.summary_lm(fit)

r2 = summary[7][0]
ar2 = summary[8][0]
ftest = summary[9][0]

# openturns like
basis = ot.Basis([f])
model = ot.LinearModelAlgorithm(X, basis, Y)
model.run()
result = model.getResult()
analysis = ot.LinearModelAnalysis(result)

np.testing.assert_almost_equal(result.getRSquared(), r2, 14)
np.testing.assert_almost_equal(result.getAdjustedRSquared(), ar2, 14)
np.testing.assert_almost_equal(analysis.getFisherScore(), ftest, 13)

# ---------------------------------------------------------------------
# Model 5 : only intercept

formula = Formula("sr ~ 1")
fit = stats.lm(formula)
summary = stats.summary_lm(fit)

"""
list(summary.names) provides
['call', 'terms', 'residuals', 'coefficients', 'aliased', 'sigma', 'df', 'r.squared', 'adj.r.squared', 'cov.unscaled']
 """

r2 = summary[7][0]
ar2 = summary[8][0]

basis = ot.Basis([h])
model = ot.LinearModelAlgorithm(X, basis, Y)
model.run()
result = model.getResult()
analysis = ot.LinearModelAnalysis(result)

np.testing.assert_almost_equal(result.getRSquared(), r2, 14)
np.testing.assert_almost_equal(result.getAdjustedRSquared(), ar2, 14)



"""
Gaussian Process Regression: use an isotropic covariance kernel
===============================================================

In typical machine learning applications, Gaussian process regression
surrogate models (refer to :ref:`gaussian_process_regression`) take several inputs,
and those inputs are usually heterogeneous
(e.g. in the :doc:`cantilever beam
</auto_surrogate_modeling/gaussian_process_regression/plot_gpr_cantilever_beam>` use case,
inputs are various physical quantities).

In geostatistical applications however, inputs are typically spatial
coordinates, which means one can assume the output varies at the same rate
in all directions.
This calls for a specific kind of covariance kernel, represented
in the library by the :class:`~openturns.IsotropicCovarianceModel` class.
"""

# TODO : change reference to plot_gpr_cantilever_beam

# %%
# Modeling temperature across a surface
# -------------------------------------
# In this example, we collect temperature data over a floorplan using sensors.

# %%
import numpy as np
import openturns as ot
import matplotlib.pyplot as plt


coordinates = ot.Sample(
    [
        [100.0, 100.0],
        [500.0, 100.0],
        [900.0, 100.0],
        [100.0, 350.0],
        [500.0, 350.0],
        [900.0, 350.0],
        [100.0, 600.0],
        [500.0, 600.0],
        [900.0, 600.0],
    ]
)
observations = ot.Sample(
    [[25.0], [25.0], [10.0], [20.0], [25.0], [20.0], [15.0], [25.0], [25.0]]
)

# %%
# Let us plot the data.

# Extract coordinates.
x = np.array(coordinates[:, 0])
y = np.array(coordinates[:, 1])

# Plot the data with a scatter plot and a color map.
fig = plt.figure()
plt.scatter(x, y, c=observations, cmap="viridis")
plt.colorbar()
plt.show()

# %%
# Because we are going to view several Gaussian Process Regression models in this example,
# we use a function to automate the process of optimizing the scale parameter
# and producing the metamodel.

lower = 50.0
upper = 1000.0


def fitGPR(coordinates, observations, covarianceModel, basis):
    """
    Fit the parameters of a Gaussian Process Regression surrogate model.
    """
    # Set the starting scale for the optimization.
    scale_dimension = covarianceModel.getScale().getDimension()
    covarianceModel.setScale([upper] * scale_dimension)

    # Prepare to fit Gaussian process hyperparameters.
    fitter = ot.GaussianProcessFitter(coordinates, observations, covarianceModel, basis)

    # Fit the GP hyperparameters.
    fitter.run()
    fitter_result = fitter.getResult()

    # Based on the GP hyperparameters perform the regression.
    regression = ot.GaussianProcessRegression(fitter_result)
    regression.run()
    result = regression.getResult()
    surrogate = result.getMetaModel()
    return result, surrogate


# %%
# Let us define a helper function to plot Gaussian Process Regression predictions.


def plotGPRPredictions(gprModel):
    """
    Plot the predictions of a Gaussian Process Regression surrogate model.
    """
    # Create the mesh of the box [0., 1000.] * [0., 700.]
    myInterval = ot.Interval([0.0, 0.0], [1000.0, 700.0])

    # Define the number of intervals in each direction of the box
    nx = 20
    ny = 20
    myIndices = [nx - 1, ny - 1]
    myMesher = ot.IntervalMesher(myIndices)
    myMeshBox = myMesher.build(myInterval)

    # Predict
    vertices = myMeshBox.getVertices()
    predictions = gprModel(vertices)

    # Format for plot
    X = np.array(vertices[:, 0]).reshape((ny, nx))
    Y = np.array(vertices[:, 1]).reshape((ny, nx))
    predictions_array = np.array(predictions).reshape((ny, nx))

    # Plot
    plt.figure()
    plt.pcolormesh(X, Y, predictions_array, shading="auto")
    plt.colorbar()
    plt.show()
    return


# %%
# Predict with an anisotropic geometric covariance kernel
# -------------------------------------------------------
# In order to illustrate the usefulness of isotropic covariance kernels,
# we first perform prediction with an anisotropic geometric kernel.

# %%
# Keep in mind that, when there are more than one input dimension,
# the :class:`~openturns.CovarianceModel` classes use a multidimensional
# scale parameter :math:`\vect{\theta}`.
# They are anisotropic geometric by default.
#
# Our example has two input dimensions,
# so :math:`\vect{\theta} = (\theta_1, \theta_2)`.


inputDimension = 2
basis = ot.ConstantBasisFactory(inputDimension).build()
covarianceModel = ot.SquaredExponential(inputDimension)
gpr_result, surrogate_model = fitGPR(coordinates, observations, covarianceModel, basis)
plotGPRPredictions(surrogate_model)

# %%
# We see weird vertical columns on the plot.
# How did this happen? Let us have a look at the optimized scale parameter
# :math:`\hat{\vect{\theta}} = (\hat{\theta}_1, \hat{\theta}_2)`.

print(gpr_result.getCovarianceModel().getScale())
# %%
# The value of :math:`\hat{\theta}_1` is actually equal to the lower bound:

print(lower)

# %%
# This means that temperatures are likely to vary a lot along the X axis
# and much slower across the Y axis based on the observation data.

# %%
# Predict with an isotropic covariance kernel
# ---------------------------------------------------
# If we know that variations of the temperature are isotropic
# (i.e. with no priviledged direction),
# we can embed this information within the covariance kernel.

isotropic = ot.IsotropicCovarianceModel(ot.SquaredExponential(), inputDimension)

# %%
# The :class:`~openturns.IsotropicCovarianceModel` class creates an isotropic
# version with a given input dimension of a :class:`~openturns.CovarianceModel`.
# Because is isotropic, it only needs one scale parameter :math:`\theta_{iso}`
# and it will make sure :math:`\theta_1 = \theta_2 = \theta_{iso}` at all times
# during the optimization.

gpr_result, surrogate_model = fitGPR(coordinates, observations, isotropic, basis)
print(gpr_result.getCovarianceModel().getScale())

# %%
# Prediction with the isotropic covariance kernel is much more satisfactory.

# sphinx_gallery_thumbnail_number = 3
plotGPRPredictions(surrogate_model)



"""
Draw an histogram
=================
"""

# %%
# In this example we are going to a plot a histogram from a unidimensional sample.

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# Create data to visualize
size = 500
normal = ot.Normal(1)
sample = normal.getSample(size)

# %%
# Draw histogram
graph = ot.HistogramFactory().build(sample).drawPDF()
view = otv.View(graph)

# %%
otv.View.ShowAll()



"""
Subset Sampling
===============
"""

# %%
#
# The objective is to evaluate a probability from the Subset sampling technique.
#
# We consider the function :math:`g : \mathbb{R}^2 \rightarrow \mathbb{R}` defined by:
#
# .. math::
#   \begin{align*}
#   g(X)= 20-(x_1-x_2)^2-8(x_1+x_2-4)^3
#   \end{align*}
#
# and the input random vector :math:`X = (X_1, X_2)` which follows a Normal distribution with independent components,
# and identical marginals with 0.25 mean and unit variance:
#
# .. math::
#   \begin{align*}
#   X \sim  \mathcal{N}(\mu = [0.25, 0.25], \sigma = [1,1], cor = I_2)
#   \end{align*}
#
# We want to evaluate the probability:
#
# .. math::
#   \begin{align*}
#   p = \mathbb{P} \{ g(X) \leq 0 \}
#   \end{align*}
#

# %%
# First, import the python modules:

# %%
import openturns as ot
import openturns.viewer as otv

# %%
# Create the probabilistic model :math:`Y = g(X)`
# -----------------------------------------------

# %%
# Create the input random vector :math:`X`:
X = ot.RandomVector(ot.Normal([0.25] * 2, [1] * 2, ot.IdentityMatrix(2)))

# %%
# Create the function :math:`g`:
g = ot.SymbolicFunction(["x1", "x2"], ["20-(x1-x2)^2-8*(x1+x2-4)^3"])
print("function g: ", g)

# %%
# Create the output random vector :math:`Y = g(X)`:
Y = ot.CompositeRandomVector(g, X)

# %%
# Create the event :math:`\{ Y = g(X) \leq 0 \}`
# ----------------------------------------------
event = ot.ThresholdEvent(Y, ot.Less(), 0.0)

# %%
# Evaluate the probability with the subset sampling technique
# -----------------------------------------------------------

# %%
algo = ot.SubsetSampling(event)

# %%
# In order to get all the inputs and outputs that realize the event, you have to mention it now:
algo.setKeepSample(True)

# %%
# Now you can run the algorithm!
algo.run()

# %%
result = algo.getResult()
proba = result.getProbabilityEstimate()
print("Proba Subset = ", proba)
print("Current coefficient of variation = ", result.getCoefficientOfVariation())

# %%
# The confidence interval of level :math:`95\%` is:
ci95 = result.getProbabilityDistribution().computeBilateralConfidenceInterval(0.95)
print("Confidence interval (0.95) = ", ci95)

# %%
# You can also get the successive thresholds used by the algorithm:
levels = algo.getThresholdPerStep()
print("Levels of g = ", levels)

# %%
# Draw the subset samples used by the algorithm
# ---------------------------------------------

# %%
# You can get the number :math:`N_s` of steps with:
Ns = algo.getStepsNumber()
print("Number of steps= ", Ns)

# %%
# Get all the inputs where :math:`g` was evaluated at each step
list_subSamples = list()
for step in range(Ns):
    list_subSamples.append(algo.getInputSample(step))

# %%
# The following graph draws each subset sample and the frontier :math:`g(x_1, x_2) = l_i` where :math:`l_i` is the threshold at the step :math:`i`:
graph = ot.Graph()
graph.setAxes(True)
graph.setGrid(True)
graph.setXTitle(r"$x_1$")
graph.setYTitle(r"$x_2$")

# %%
# Add all the subset samples:
for i in range(Ns):
    cloud = ot.Cloud(list_subSamples[i])
    cloud.setPointStyle("dot")
    graph.add(cloud)

# %%
# Add the frontiers :math:`g(x_1, x_2) = l_i` where :math:`l_i` is the threshold at the step :math:`i`:
gIsoLines = g.draw([-3] * 2, [5] * 2, [128] * 2)
dr = gIsoLines.getDrawable(0)
dr.setColor("black")
for i in range(levels.getSize()):
    dr.setLevels([levels[i]])
    dr.setLegend(r"$g(X) = $" + str(round(levels[i], 2)))
    graph.add(dr)

# %%
_ = otv.View(graph)

# %%
# Draw the frontiers only
# -----------------------

# %%
# The following graph enables to understand the progression of the algorithm:
graph = ot.Graph()
graph.setAxes(True)
graph.setGrid(True)
dr = gIsoLines.getDrawable(0)
colors = ot.Drawable().BuildDefaultPalette(len(levels))
for i in range(levels.getSize()):
    dr.setLevels([levels[i]])
    dr.setLegend(r"$g(X) = $" + str(round(levels[i], 2)))
    dr.setColor(colors[i])
    graph.add(dr)

graph.setLegendPosition("lower left")
graph.setTitle("Subset sampling: thresholds")
graph.setXTitle(r"$x_1$")
graph.setYTitle(r"$x_2$")

_ = otv.View(graph)

# %%
# Get all the input and output points that realized the event
# -----------------------------------------------------------

# %%
# The following lines are possible only if you have mentioned that you wanted to keep samples with the method *algo.setKeepSample(True)*
select = ot.SubsetSampling.EVENT1  # points that realize the event
step = Ns - 1  # get the working sample from last iteration
inputEventSample = algo.getInputSample(step, select)
outputEventSample = algo.getOutputSample(step, select)
print("Number of event realizations = ", inputEventSample.getSize())

# %%
# Draw them! They are all in the event space.
graph = ot.Graph()
graph.setAxes(True)
graph.setGrid(True)
cloud = ot.Cloud(inputEventSample)
cloud.setPointStyle("dot")
graph.add(cloud)
gIsoLines = g.draw([-3] * 2, [5] * 2, [1000] * 2)
dr = gIsoLines.getDrawable(0)
dr.setLevels([0.0])
dr.setColor("red")
graph.add(dr)
_ = otv.View(graph)

# %%
otv.View.ShowAll()



import openturns as ot
from matplotlib import pyplot as plt
import openturns.viewer as otv


mu = [2.0, 2.0]
sigma = [1.0, 1.0]
R = ot.CorrelationMatrix(2)
R[0, 1] = 0.8
myNormal1 = ot.Normal(mu, sigma, R)
R2 = ot.CorrelationMatrix(2)
R2[0, 1] = -0.8
mu2 = [3.0, 3.0]
myNormal2 = ot.Normal(mu2, sigma, R2)

myMixture = ot.Mixture([myNormal1, myNormal2], [0.3, 0.7])
myMixture.setDescription(["$x_1$", "$x_2$"])

graphPDF = myMixture.drawPDF()
graphCDF = myMixture.drawCDF()

fig = plt.figure(figsize=(8, 4))
pdf_axis = fig.add_subplot(121)
cdf_axis = fig.add_subplot(122)
pdf_axis.set_xlim(auto=True)
cdf_axis.set_xlim(auto=True)

otv.View(graphPDF, figure=fig, axes=[pdf_axis], add_legend=True)
otv.View(graphCDF, figure=fig, axes=[cdf_axis], add_legend=True)
fig.suptitle("Mixture: 0.3*Normal1 + 0.7*Normal2: pdf and cdf")



#! /usr/bin/env python

import openturns as ot
import math as m

ot.TESTPREAMBLE()


inputDimension = 3
outputDimension = 1

inputName = ["X1", "X2", "X3"]

# Test with Ishigami function
formulaIshigami = ot.Description(outputDimension)
formulaIshigami[0] = (
    "sin(pi_*X1)+7*sin(pi_*X2)*sin(pi_*X2)+0.1*((pi_*X3)*(pi_*X3)*(pi_*X3)*(pi_*X3))*sin(pi_*X1)"
)

modelIshigami = ot.SymbolicFunction(inputName, formulaIshigami)

distributions = ot.JointDistribution([ot.Uniform(-1.0, 1.0)] * inputDimension)

sensitivityAnalysis = ot.FAST(modelIshigami, distributions, 400)

firstOrderIndices = sensitivityAnalysis.getFirstOrderIndices()
totalOrderIndices = sensitivityAnalysis.getTotalOrderIndices()

# Comparaison with reference analytical values
a = 7.0
b = 0.1
covTh = (b**2 * m.pi**8) / 18.0 + (b * m.pi**4) / 5.0 + (a**2) / 8.0 + 1.0 / 2.0
sob_1 = [
    (b * m.pi**4 / 5.0 + b**2 * m.pi**8 / 50.0 + 1.0 / 2.0) / covTh,
    (a**2 / 8.0) / covTh,
    0.0,
]
sob_2 = [0.0, (b**2 * m.pi**8 / 18.0 - b**2 * m.pi**8 / 50.0) / covTh, 0.0]
sob_3 = [0.0]
sob_T1 = [
    sob_1[0] + sob_2[0] + sob_2[1] + sob_3[0],
    sob_1[1] + sob_2[0] + sob_2[2] + sob_3[0],
    sob_1[2] + sob_2[1] + sob_2[2] + sob_3[0],
]

for i in range(inputDimension):
    value = firstOrderIndices[i]
    print(
        "Ishigami first order FAST indice ",
        i,
        "= %.8f" % value,
        "absolute error=%.8f" % abs(value - sob_1[i]),
    )
print("\n")
for i in range(inputDimension):
    value = totalOrderIndices[i]
    print(
        "Ishigami total order FAST indice",
        i,
        "= %.8f" % value,
        "absolute error=%.8f" % abs(value - sob_T1[i]),
    )

# Test with G-Sobol function
formulaGSobol = ["1.0"]
covTh = 1.0
a = ot.Point(inputDimension)
for i in range(inputDimension):
    a[i] = 0.5 * i
    covTh = covTh * (1.0 + 1.0 / (3.0 * (1.0 + a[i]) ** 2))
    formulaGSobol[0] = (
        formulaGSobol[0]
        + " * ((abs(4.0 * X"
        + str(i + 1)
        + " - 2.0) + "
        + str(a[i])
        + ") / (1.0 + "
        + str(a[i])
        + "))"
    )

covTh = covTh - 1.0
modelGSobol = ot.SymbolicFunction(inputName, formulaGSobol)

distributions = ot.JointDistribution([ot.Uniform(0.0, 1.0)] * inputDimension)

sensitivityAnalysis = ot.FAST(modelGSobol, distributions, 400)

# Comparaison with reference analytical values
firstOrderIndices = sensitivityAnalysis.getFirstOrderIndices()
totalOrderIndices = sensitivityAnalysis.getTotalOrderIndices()
# First-order indices
V_i = ot.Point(inputDimension)
Vtot_i = ot.Point(inputDimension)
prod_V_i = 1.0
for i in range(inputDimension):
    V_i[i] = 1.0 / (3.0 * (1.0 + a[i]) ** 2.0)
    prod_V_i *= V_i[i]
# Total indices
Vtot_i[0] = V_i[0] + V_i[0] * V_i[1] + V_i[0] * V_i[2] + prod_V_i
Vtot_i[1] = V_i[1] + V_i[0] * V_i[1] + V_i[1] * V_i[2] + prod_V_i
Vtot_i[2] = V_i[2] + V_i[0] * V_i[2] + V_i[1] * V_i[2] + prod_V_i
# Results
print("\n")
for i in range(inputDimension):
    value = firstOrderIndices[i]
    print(
        "G-Sobol first order FAST indice ",
        i,
        "= %.8f" % value,
        "absolute error=%.8f" % abs(value - V_i[i] / covTh),
    )

print("\n")
for i in range(inputDimension):
    value = totalOrderIndices[i]
    print(
        "G-Sobol total order FAST indices ",
        i,
        "= %.8f" % value,
        "absolute error=%.8f" % abs(value - Vtot_i[i] / covTh),
    )



r"""
Generate observations of the Chaboche mechanical model
======================================================

In this example we present the simulation of noisy observations of the Chaboche model.
A detailed explanation of this mechanical law is presented :ref:`here <use-case-chaboche>`.
We show how to produce the observations that we use in the calibration
model of :doc:`Calibration of the Chaboche model
</auto_calibration/least_squares_and_gaussian_calibration/plot_calibration_chaboche>`.

Parameters to calibrate
-----------------------

The vector of parameters to calibrate is:

.. math::

   \theta = (R,C,\gamma).


The true values of the parameters are:

- :math:`R = 750\times 10^6`,
- :math:`C = 2750\times 10^6`,
- :math:`\gamma = 10`.

This is the set of *true* values that we wish to estimate with the calibration
methods.
In practical studies, these values are unknown.

Observations
------------

In order to create a calibration problem, we make the hypothesis that the
strain has the following distribution:

.. math::
   \epsilon \sim \mathcal{U}(0,0.07).

where :math:`\mathcal{U}` is the uniform distribution.
Moreover, we consider a Gaussian noise on the observed constraint:

.. math::
   \epsilon_\sigma \sim \mathcal{N} \left(0,10\times 10^6\right)


and we make the hypothesis that the observation errors are independent.
We set the number of observations to:

.. math::

   n = 10.


We generate a Monte-Carlo sample with size :math:`n`:

.. math::

   \sigma_i = g(\epsilon_i,R,C,\gamma) + (\epsilon_\sigma)_i,


for :math:`i = 1,..., n`.
The observations are the pairs :math:`\{(\epsilon_i,\sigma_i)\}_{i=1,...,n}`,
i.e. each observation is a couple made of the strain and the corresponding stress.

Variables
---------

In the particular situation where we want to calibrate this model, the following
list presents which variables are observed input variables, input calibrated
variables and observed output variables.

- :math:`\epsilon`: Input. Observed.
- :math:`R`, :math:`C`, :math:`\gamma` : Inputs. Calibrated.
- :math:`\sigma`: Output. Observed.
"""

# %%
import openturns as ot
import openturns.viewer as otv
from openturns.usecases import chaboche_model


# %%
# Generate the observations
# -------------------------
# In practice, we generally use a data set which has been obtained from
# measurements.
# In this example, we generate the data using noisy observations of the
# physical model.
# In the next part, we will calibrate the parameters using the calibration
# algorithms.

# %%
# Load the Chaboche model

# %%
cm = chaboche_model.ChabocheModel()
print("Inputs:", cm.model.getInputDescription())
print("Outputs:", cm.model.getOutputDescription())

# %%
# We see that there are four inputs: `Strain`, `R`, `C` and `Gamma` and one
# output: `Sigma`.
# The `Strain` is observed on input and the stress `Sigma` is observed on
# output.
# Using these observations, we want to estimate the parameters `R`, `C`
# and `Gamma`.

# %%
# We get the Chaboche model and the joint input distribution :

# %%
inputDistribution = cm.distribution
print("inputDistribution:")
inputDistribution

# %%
# Set the calibration parameters
# ------------------------------
# In this part, we begin the calibration study.
# Define the value of the reference values of the :math:`\theta` parameter.
# In the Bayesian framework, this is called the mean of the *prior* Gaussian
# distribution. In the data assimilation framework, this is called the *background*.

# %%
thetaTrue = [cm.trueR, cm.trueC, cm.trueGamma]
print("theta True = ")
print("    R = %.2f (MPa)" % (cm.trueR / 1.0e6))
print("    C = %.2f (MPa)" % (cm.trueC / 1.0e6))
print("    Gamma = %.4f" % (cm.trueGamma))


# %%
# The following statement create the calibrated function from the model.
# The calibrated parameters `R`, `C`, `Gamma` are at indices 1, 2, 3 in
# the inputs arguments of the model.

# %%
calibratedIndices = [1, 2, 3]
mycf = ot.ParametricFunction(cm.model, calibratedIndices, thetaTrue)

# %%
# Create a regular grid of the strains and evaluate the corresponding outputs.

# %%
nbobs = 10
step = (cm.strainMax - cm.strainMin) / (nbobs - 1)
rg = ot.RegularGrid(cm.strainMin, step, nbobs)
observedStrain = rg.getVertices()
predictedStress = mycf(observedStrain)

# %%
# Generate observation noise.
# Here we consider a Gaussian observation noise, that we add
# to the output of the model.

# %%
stressObservationNoiseSigma = 10.0e6  # (Pa)
noiseSigma = ot.Normal(0.0, stressObservationNoiseSigma)
sampleNoiseStress = noiseSigma.getSample(nbobs)
observedStress = predictedStress + sampleNoiseStress

# %%
# Gather the data into a sample.

# %%
data = ot.Sample(nbobs, 2)
data[:, 0] = observedStrain
data[:, 1] = observedStress
data.setDescription(["Strain", "Stress (Pa)"])
data

# %%
# Then we plot the model and compare it to the observations.

# %%
graph = ot.Graph("Model with true parameters", "Strain", "Stress (MPa)")
# Plot the model
curve = mycf.draw(cm.strainMin, cm.strainMax, 50).getDrawable(0)
curve.setLegend("True model")
curve.setLineStyle(ot.ResourceMap.GetAsString("CalibrationResult-ObservationLineStyle"))
graph.add(curve)
# Plot the noisy observations
cloud = ot.Cloud(observedStrain, observedStress)
cloud.setLegend("Observations")
cloud.setPointStyle(
    ot.ResourceMap.GetAsString("CalibrationResult-ObservationPointStyle")
)
graph.add(cloud)
graph.setLegendPosition("upper left")
view = otv.View(graph)

# %%
# We see that the observations are relatively noisy, but that the
# trend is clear: this shows that it may be possible to fit the model.
# At this point, we have a data set that we can use for calibration
# and a model to calibrate.

otv.View.ShowAll()



"""
A quick start guide to graphs
==============================
"""

# sphinx_gallery_thumbnail_number = 4
# %%
#
# In this example, we show how to create graphs. We show how to create and configure its axes and its colors. We show how to create a plot based on the combination of several plots.

# %%
# The `draw` method the `Graph` class
# -----------------------------------
#
# The simplest way to create a graph is to use the `draw` method. The :class:`~openturns.Normal` distribution for example provides a method to draw its density function.

# %%
import openturns as ot
import openturns.viewer as otv
import matplotlib.pyplot as plt


# %%
n = ot.Normal()
n

# %%
graph = n.drawPDF()
view = otv.View(graph)

# %%
# To configure the look of the plot, we can first observe the type of graph returned by the `drawPDF` method returns: it is a :class:`~openturns.Graph`.

# %%
graph = n.drawPDF()
type(graph)

# %%
# The class:`~openturns.Graph` class provides several methods to configure the legends, the title and the colors.
# Since a graph  can contain several sub-graphs, the `setColors` method takes a list of colors as inputs argument: each item of the list corresponds to the sub-graphs.

# %%
graph.setXTitle("N")
graph.setYTitle("PDF")
graph.setTitle("Probability density function of the standard Gaussian distribution")
graph.setLegends(["N"])
graph.setColors(["blue"])
view = otv.View(graph)

# %%
# Combine several graphs
# ----------------------
#
# In order to combine several graphs, we can use the `add` method.

# %%
# Let us create an empirical histogram from a sample.

# %%
sample = n.getSample(100)

# %%
histo = ot.HistogramFactory().build(sample).drawPDF()
view = otv.View(histo)

# %%
# Then we add the histogram to the `graph` with the `add` method. The `graph` then contains two plots.

# %%
graph.add(histo)
view = otv.View(graph)

# %%
# Draw a cloud
# ------------
#
# The :class:`~openturns.Cloud` class creates clouds of bidimensional points. To illustrate it, let us create two Normal distributions in two dimensions.

# %%
# Create a Funky distribution
corr = ot.CorrelationMatrix(2)
corr[0, 1] = 0.2
copula = ot.NormalCopula(corr)
x1 = ot.Normal(-1.0, 1)
x2 = ot.Normal(2, 1)
x_funk = ot.JointDistribution([x1, x2], copula)

# %%
# Create a Punk distribution
x1 = ot.Normal(1.0, 1)
x2 = ot.Normal(-2, 1)
x_punk = ot.JointDistribution([x1, x2], copula)

# %%
# Let us mix these two distributions.

# %%
mixture = ot.Mixture([x_funk, x_punk], [0.5, 1.0])

# %%
n = 500
sample = mixture.getSample(n)

# %%
graph = ot.Graph(f"{n=}", "X1", "X2")
cloud = ot.Cloud(sample)
graph.add(cloud)
view = otv.View(graph)

# %%
# We sometimes want to customize the graph by choosing the type of point (square, triangle, circle, etc.), of line (continuous, dashed, etc.) or another parameter.
# We can know the list of possible values with the corresponding `getValid` method.
#
# For example, the following function returns the possible values of the `PointStyle` parameter.

# %%
ot.Drawable.GetValidPointStyles()

# %%
# The following method returns the list of colors.

# %%
ot.Drawable.GetValidColors()[0:10]

# %%
# In the following graph, we use the "aquamarine1" color with "fcircle" circles.

# %%
graph = ot.Graph("n=%d" % (n), "X1", "X2")
cloud = ot.Cloud(sample)
cloud.setColor("aquamarine1")
cloud.setPointStyle("fcircle")
graph.add(cloud)
view = otv.View(graph)

# %%
# Configure the style of points and the thickness of a curve
# ----------------------------------------------------------
#
# Assume that we want to plot the `sine` curve from -2 to 2. The simplest way is to use the `draw` method of the function.

# %%
g = ot.SymbolicFunction("x", "sin(x)")

# %%
graph = g.draw(-2, 2)
view = otv.View(graph)

# %%
# One would rather get a dashed curve: let us search for the available line styles.

# %%
ot.Drawable.GetValidLineStyles()


# %%
# In order to use the :class:`~openturns.Curve` class, it will be easier if we have a method to generate a :class:`~openturns.Sample` containing points regularly spaced in an interval.


# %%
def linearSample(xmin, xmax, npoints):
    """Returns a sample created from a regular grid
    from xmin to xmax with npoints points."""
    step = (xmax - xmin) / (npoints - 1)
    rg = ot.RegularGrid(xmin, step, npoints)
    vertices = rg.getVertices()
    return vertices


# %%
x = linearSample(-2, 2, 50)
y = g(x)

# %%
graph = ot.Graph("Sinus", "x", "sin(x)")
curve = ot.Curve(x, y)
curve.setLineStyle("dashed")
curve.setLineWidth(4)
graph.add(curve)
view = otv.View(graph)


# %%
# Create colored curves
# ---------------------
#
# In some situations, we want to create curves with different colors.
# In this case, the following function generates a color corresponding to the `indexCurve` integer in a ensemble of `maximumNumberOfCurves` curves.


# %%
def createHSVColor(indexCurve, maximumNumberOfCurves):
    """Create a HSV color for the indexCurve-th curve
    from a sample with maximum size equal to maximumNumberOfCurves"""
    color = ot.Drawable.ConvertFromHSV(
        indexCurve * 360.0 / maximumNumberOfCurves, 1.0, 1.0
    )
    return color


# %%
pofa = ot.HermiteFactory()

# %%
graph = ot.Graph("Orthonormal Hermite polynomials", "x", "y")
graph.setLegendPosition("lower right")
degreemax = 5
for k in range(degreemax):
    pk = pofa.build(k)
    curve = pk.draw(-3.0, 3.0, 50)
    curve.setLegends(["P%d" % (k)])
    curve.setColors([createHSVColor(k, degreemax)])
    graph.add(curve)
view = otv.View(graph)

# %%
# Create matrices of graphs
# -------------------------
#
# The library provides features to create a grid of graphs. However, we can use the `add_subplot` function from `Matplotlib`.
#
# Let us create two graphs of the PDF and CDF of the following Normal distribution..

# %%
n = ot.Normal()
myPDF = n.drawPDF()
myCDF = n.drawCDF()

# %%
# Using `~openturns.GridLayout`.
grid = ot.GridLayout(1, 2)
grid.setGraph(0, 0, myPDF)
grid.setGraph(0, 1, myCDF)
_ = otv.View(grid)

# %%
# Another method is to create a figure with the `figure` function from `Matplotlib`,
# then add two graphs with the `add_subplot` function.
# We use the `otv.View` function to create the required `Matplotlib` object.
# Since we are not interested by the output of the `View` function, we use the dummy variable `_` as output.
# The title is finally configured with `suptitle`.

# %%
fig = plt.figure(figsize=(12, 4))
ax_pdf = fig.add_subplot(1, 2, 1)
_ = otv.View(myPDF, figure=fig, axes=[ax_pdf])
ax_cdf = fig.add_subplot(1, 2, 2)
_ = otv.View(myCDF, figure=fig, axes=[ax_cdf])
_ = fig.suptitle("The gaussian")

# %%
# Save a plot into a file
# -----------------------

# %%
# The :class:`openturns.viewer.View` class has a `save` method which saves the graph into an image.

# %%

# %%
n = ot.Normal()
graph = n.drawPDF()
view = otv.View(graph)
view.save("normal.png")

# %%
# We can use the `dpi` option to configure the resolution in dots per inch.

# %%
view.save("normal-100dpi.png", dpi=100)

# %%
# Configure the size of a graph with `matplotlib`
# -----------------------------------------------

# %%

# %%
# We first create a graph containing the PDF of a Normal distribution

# %%
n = ot.Normal()
graph = n.drawPDF()

# %%
# The `figure_kw` keyword argument sets the optional arguments of the figure. In the following statement, we set the figure size in inches

# %%
view = otv.View(graph, figure_kw={"figsize": (12, 8)})

# %%
# The `getFigure` method returns the current figure. This allows one to configure it as any other Matplotlib figure. In the following example, we configure the `suptitle`.

# %%
fig = view.getFigure()
fig.suptitle("The suptitle")
fig

# %%
# The `plot_kw` optional argument sets the arguments of the plot. In the following example, we set the color of the plot in blue.

# %%
view = otv.View(graph, plot_kw={"color": "blue"})

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

marginals = [ot.Normal(), ot.Uniform()]
R = ot.CorrelationMatrix(2, [1.0, 0.5, 0.5, 1.0])
copula = ot.NormalCopula(R)
factories = [
    ot.SoizeGhanemFactory(ot.JointDistribution(marginals)),
    ot.SoizeGhanemFactory(ot.JointDistribution(marginals, copula), False),
    ot.SoizeGhanemFactory(ot.JointDistribution(marginals, copula), True),
]
x = [0.5] * 2
kMax = 5
ot.ResourceMap.SetAsUnsignedInteger("IteratedQuadrature-MaximumSubIntervals", 2048)
ot.ResourceMap.SetAsScalar("IteratedQuadrature-MaximumError", 1.0e-6)
for soize in factories:
    distribution = soize.getMeasure()
    print("SoizeGhanem=", soize)
    functions = list()
    for k in range(kMax):
        functions.append(soize.build(k))
        print("SoizeGhanem(", k, ")=", functions[k].getEvaluation())
        print("SoizeGhanem(", k, ")(", x, "=", functions[k](x))
    M = ot.SymmetricMatrix(kMax)
    for m in range(kMax):
        for n in range(m + 1):

            def wrapper(x):
                return functions[m](x) * functions[n](x)[0] * distribution.computePDF(x)

            kernel = ot.PythonFunction(distribution.getDimension(), 1, wrapper)
            value = ot.IteratedQuadrature().integrate(kernel, distribution.getRange())[
                0
            ]
            if abs(value) >= 1.0e-6:
                M[m, n] = value
    print("M=\n", M)

# Test isTensorProduct(), case True
sgTensorProduct = ot.SoizeGhanemFactory(ot.JointDistribution(marginals))
assert sgTensorProduct.isTensorProduct()
# Test isTensorProduct(), case False
sgTensorProduct = ot.SoizeGhanemFactory(ot.JointDistribution(marginals, copula))
assert not sgTensorProduct.isTensorProduct()

# Test getMarginal
basis = ot.SoizeGhanemFactory(ot.JointDistribution(marginals))
basisMarginal = basis.getMarginal([0])
for i in range(20):
    function = basisMarginal.build(i)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()
ot.PlatformInfo.SetNumericalPrecision(5)


def cleanSymmetricTensor(inSymmetricTensor):
    rowDim = inSymmetricTensor.getNbRows()
    colDim = inSymmetricTensor.getNbColumns()
    sheetDim = inSymmetricTensor.getNbSheets()
    for i in range(rowDim):
        for j in range(colDim):
            for k in range(sheetDim):
                if abs(inSymmetricTensor[i, j, k]) < 1.0e-6:
                    inSymmetricTensor[i, j, k] = 0.0
    return inSymmetricTensor


# Instantiate one distribution object
dim = 3
meanPoint = ot.Point(dim, 1.0)
meanPoint[0] = 0.5
meanPoint[1] = -0.5
sigma = ot.Point(dim, 1.0)
sigma[0] = 2.0
sigma[1] = 3.0
R = ot.CorrelationMatrix(dim)
for i in range(1, dim):
    R[i, i - 1] = 0.5
distribution = ot.Normal(meanPoint, sigma, R)

# Test for sampling
size = 10000
sample = distribution.getSample(size)
print("sample first=", repr(sample[0]), " last=", repr(sample[size - 1]))
print("sample mean=", repr(sample.computeMean()))
print("sample covariance=", repr(sample.computeCovariance()))

transform = distribution.getIsoProbabilisticTransformation()
print("isoprobabilistic transformation=", repr(transform))
transformedSample = transform(sample)
print(
    "transformed sample first=",
    repr(transformedSample[0]),
    " last=",
    repr(transformedSample[size - 1]),
)
print("transformed sample mean=", repr(transformedSample.computeMean()))
print("transformed sample covariance=", repr(transformedSample.computeCovariance()))

# Test for evaluation
inverseTransform = distribution.getInverseIsoProbabilisticTransformation()
print("inverse isoprobabilistic transformation=", repr(inverseTransform))
transformedBackSample = inverseTransform(transformedSample)
print(
    "transformed back sample first=",
    repr(transformedBackSample[0]),
    " last=",
    repr(transformedBackSample[size - 1]),
)
print("transformed back sample mean=", repr(transformedBackSample.computeMean()))
print(
    "transformed back sample covariance=",
    repr(transformedBackSample.computeCovariance()),
)
point = ot.Point(dim, 1.0)
print("point=", repr(point))
transformedPoint = transform(point)
print("transform value at point        =", repr(transformedPoint))
print("transform gradient at point     =", repr(transform.gradient(point)))
print(
    "transform gradient at point (FD)=",
    repr(
        ot.CenteredFiniteDifferenceGradient(1.0e-5, transform.getEvaluation()).gradient(
            point
        )
    ),
)
print(
    "transform hessian at point      =",
    repr(cleanSymmetricTensor(transform.hessian(point))),
)
print(
    "transform hessian at point (FD) =",
    repr(
        cleanSymmetricTensor(
            ot.CenteredFiniteDifferenceHessian(
                1.0e-4, transform.getEvaluation()
            ).hessian(point)
        )
    ),
)
print(
    "inverse transform value at transformed point        =",
    repr(inverseTransform(transformedPoint)),
)
print(
    "inverse transform gradient at transformed point (FD)=",
    repr(inverseTransform.gradient(transformedPoint)),
)
print(
    "inverse transform gradient at transformed point     =",
    repr(
        ot.CenteredFiniteDifferenceGradient(
            1.0e-5, inverseTransform.getEvaluation()
        ).gradient(transformedPoint)
    ),
)
print(
    "inverse transform hessian at transformed point      =",
    repr(cleanSymmetricTensor(inverseTransform.hessian(transformedPoint))),
)
print(
    "inverse transform hessian at transformed point (FD) =",
    repr(
        cleanSymmetricTensor(
            ot.CenteredFiniteDifferenceHessian(
                1.0e-4, inverseTransform.getEvaluation()
            ).hessian(transformedPoint)
        )
    ),
)

# Test for parameters
print("parameters gradient at point=", repr(transform.parameterGradient(point)))



"""
Create a Gaussian process from a cov. model using HMatrix
=========================================================
"""

# %%
#
# In this basic example we build a Gaussian process from its covariance model and use the `HMatrix` method as a sampling method.
# Several methods and parameters are presented to set the HMatrix compression.
# This is an advanced topic and we should highlight key ideas only in this example.

# %%
import openturns as ot
import openturns.viewer as otv


# %%
# Definition of the covariance model
# ----------------------------------
#
# We define our Gaussian process from its covariance model. We consider a Gaussian kernel with the following parameters.

# %%
dimension = 1
amplitude = [1.0] * dimension
scale = [1] * dimension
covarianceModel = ot.GeneralizedExponential(scale, amplitude, 2)

# %%
# We define the time grid on which we want to sample the Gaussian process.

# %%
tmin = 0.0
step = 0.01
n = 10001
timeGrid = ot.RegularGrid(tmin, step, n)

# %%
# Finally we define the Gaussian process.
process = ot.GaussianProcess(covarianceModel, timeGrid)
print(process)

# %%
# Basics on the HMatrix algebra
# -----------------------------
#
# The `HMatrix` framework uses efficient linear algebra techniques to speed-up the
# (Cholesky) factorization of the covariance matrix.
# This method can be tuned with several parameters. We should concentrate on the easiest ones.

# %%
# We set the sampling method to `HMAT` (default is the classical/dense case).
process.setSamplingMethod(ot.GaussianProcess.HMAT)


# %%
# The `HMatrix` framework uses an algebraic algorithm to compress sub-blocks of the matrix. Several algorithms are available and can be set from the ResourceMap key.
ot.ResourceMap.SetAsString("HMatrix-CompressionMethod", "AcaRandom")

# %%
# There are two threshold used in the HMatrix framework. The `AssemblyEpsilon` is the most important one.
#
ot.ResourceMap.SetAsScalar("HMatrix-AssemblyEpsilon", 1e-7)
ot.ResourceMap.SetAsScalar("HMatrix-RecompressionEpsilon", 1e-7)


# %%
# Process sampling
# ----------------

# %%
# We eventually draw samples of this covariance model.
sample = process.getSample(6)
graph = sample.drawMarginal(0)
view = otv.View(graph)

# %%
# We notice here that we are able to sample the covariance model over a mesh of size `10000`, which is usually tricky on a laptop. This is mainly due to the compression.

# %%
otv.View.ShowAll()



#! /usr/bin/env python

import openturns as ot


# We create an empty ot.Sample
sample1 = ot.Sample(0, 2)
sample1.setName("Sample1")
sample1.setDescription(["a0", "a1"])
sample1.add([10.0, 20.0])
sample1.add([11.0, 21.0])
sample1.add([12.0, 22.0])
print("sample1=", repr(sample1))

# We create a new empty ot.Sample
sample2 = ot.Sample(0, 2)
sample2.setName("Sample2")
sample2.setDescription(["a0", "a1"])
sample2.add([30.0, 40.0])
sample2.add([31.0, 41.0])
sample2.add([32.0, 42.0])
print("sample2=", repr(sample2))

# Operator +|-
summation = ot.Sample(sample1 + sample2)
subtraction = ot.Sample(sample2 - sample1)
print("sample1 + sample2=", repr(summation))
print("sample2 - sample1=", repr(subtraction))

# Operator +=|-=
sample3 = ot.Sample(sample2)
sample4 = ot.Sample(sample2)
sample3 += sample1
sample4 -= sample1
print("sample3=", repr(sample3))
print("sample4=", repr(sample4))

sample5 = ot.Sample(sample2)
m = ot.SquareMatrix([[1, 2], [3, 5]])
v = ot.Point(2, 3.0)
t = ot.Point(2, 5.0)

print("sample5 =", sample5)

print("sample*2:", sample5 * 2.0)
print("2*sample:", 2.0 * sample5)
print("sample/2:", sample5 / 2.0)

print("sample*v:", sample5 * v)
print("sample/v:", sample5 / v)

# in-place
sample5 += t
print("sample+=t:", sample5)

sample5 -= t
print("sample-=t:", sample5)

sample5 *= 2.0
print("sample*=2:", sample5)

sample5 /= 2.0
print("sample/=2:", sample5)

sample5 *= v
print("sample*=v:", sample5)

sample5 /= v
print("sample/=v:", sample5)



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()


inputDimension = 3
outputDimension = 2
# Constant term
constant = ot.Matrix(inputDimension, outputDimension)
constant[0, 0] = 1.0
constant[1, 0] = 2.0
constant[2, 0] = 5.0
constant[0, 1] = 7.0
constant[1, 1] = 9.0
constant[2, 1] = 3.0

myGradient = ot.ConstantGradient(constant)
myGradient.setName("constantGradient")
inPoint = ot.Point(inputDimension)
inPoint[0] = 7.0
inPoint[1] = 8.0
inPoint[2] = 9.0
outMatrix = myGradient.gradient(inPoint)
print("myGradient=", repr(myGradient))
print(myGradient.getName(), "( ", repr(inPoint), " ) = ", repr(outMatrix))



import sympy as sp

x, a, b, a_, b_, point, gamma_, expX, beta_, alpha_, mu, sigma = sp.symbols(
    "x a b a_ b_ point gamma_ expX beta_ alpha_ mu sigma"
)

pdf = alpha_ * beta_**alpha_ / (x - gamma_) ** (alpha_ + 1)
print("dpdf/dbeta", sp.simplify(sp.diff(pdf, beta_)))
print("dpdf/dalpha", sp.simplify(sp.diff(pdf, alpha_)))
print("dpdf/dgamma", sp.simplify(sp.diff(pdf, gamma_)))


cdf = 1 - (beta_ / (x - gamma_)) ** alpha_
print("dcdf/dbeta", sp.simplify(sp.diff(cdf, beta_)))
print("dcdf/dalpha", sp.simplify(sp.diff(cdf, alpha_)))
print("dcdf/dgamma", sp.simplify(sp.diff(cdf, gamma_)))


print("ddf", sp.simplify(sp.diff(pdf, x)))



#! /usr/bin/env python

import openturns as ot

ot.TESTPREAMBLE()

# Tmin , time step and step number for TimeGrid
Tmin = 0.0
deltaT = 1.0
N = 11
timeGrid = ot.RegularGrid(Tmin, deltaT, N)

# Fixing the distribution for the WhiteNoise
dist = ot.Distribution(ot.Uniform())
print("dist = ", dist)

# Building a process from a White Noise
process = ot.Process(ot.WhiteNoise(dist))

# print the process
process.setTimeGrid(timeGrid)
print("process = ", process)

# Get a realization of the process
timeSerie = process.getRealization()
print("timeSerie = ", timeSerie)



#! /usr/bin/env python

import os
import openturns as ot

ot.PlatformInfo.SetNumericalPrecision(6)

vertices = []
vertices.append([0.0, 0.0, 0.0])
vertices.append([0.0, 0.0, 1.0])
vertices.append([0.0, 1.0, 0.0])
vertices.append([0.0, 1.0, 1.0])
vertices.append([1.0, 0.0, 0.0])
vertices.append([1.0, 0.0, 1.0])
vertices.append([1.0, 1.0, 0.0])
vertices.append([1.0, 1.0, 1.0])

simplicies = []
simplicies.append([0, 1, 2, 4])
simplicies.append([3, 5, 6, 7])
simplicies.append([1, 2, 3, 6])
simplicies.append([1, 2, 4, 6])
simplicies.append([1, 3, 5, 6])
simplicies.append([1, 4, 5, 6])

mesh3D = ot.Mesh(vertices, simplicies)

s = 3
values = ot.Normal(s).getSample(mesh3D.getVerticesNumber())
field = ot.Field(mesh3D, values)
tree = ot.KDTree(vertices)
print("field=", field)
print("input dim=", field.getInputDimension())
print("value[4]=", field.getValueAtIndex(4))
print("value[4, 0]=%.6g" % field[4, 0])
print("nearest[2]=", field.getValueAtIndex(tree.query(field[2])))
print("mesh=", field.getMesh())
print("input mean=", field.getInputMean())
print("norm=%.6g" % field.norm())
print("deformed=", field.asDeformedMesh())
print("description=", field.getDescription())

fname = "field.vtk"
field.exportToVTKFile(fname)
with open(fname) as f:
    data = f.read()
    print(data)
os.remove(fname)

scalarField = ot.Field(mesh3D, ot.Sample(len(vertices), [1.0]))
print("deformed with complement=", scalarField.asDeformedMesh([1], [0, 2, 3]))

# Default draw for 3D field (output marginal 0
graph = field.draw()
print(graph)
# Draw a given marginal
graph = field.drawMarginal(1)
print(graph)
# Draw in 3D with tuning
graph = field.draw3D(
    2,
    True,
    ot.Mesh.BuildRotationFromAngles(0.2, 0.3, 0.4),
    True,
    0.9,
    ["red", "green", "blue"],
)
print(graph)



#! /usr/bin/env python

import openturns as ot
import openturns.testing as ott
import math as m

ot.RandomGenerator.SetSeed(0)

ot.TESTPREAMBLE()


chainDim = 3
obsDim = 1
outputDim = 1

# observations
obsSize = 10
y_obs = ot.Sample(obsSize, obsDim)
y_obs[0, 0] = -9.50794871493506
y_obs[1, 0] = -3.83296694500105
y_obs[2, 0] = -2.44545713047953
y_obs[3, 0] = 0.0803625289211318
y_obs[4, 0] = 1.01898069723583
y_obs[5, 0] = 0.661725805623086
y_obs[6, 0] = -1.57581204592385
y_obs[7, 0] = -2.95308465670895
y_obs[8, 0] = -8.8878164296758
y_obs[9, 0] = -13.0812290405651
print("y_obs=", y_obs)

p = ot.Sample(obsSize, chainDim)
for i in range(obsSize):
    for j in range(chainDim):
        p[i, j] = (-2 + 5.0 * i / 9.0) ** j
print("p=", p)

fullModel = ot.SymbolicFunction(
    ["p1", "p2", "p3", "x1", "x2", "x3"], ["p1*x1+p2*x2+p3*x3", "1.0"]
)
linkFunction = ot.ParametricFunction(fullModel, range(chainDim), [0.0] * chainDim)

# instrumental distribution
instrumental = ot.Uniform(-1.0, 1.0)

# prior distribution
sigma0 = [10.0] * chainDim
Q0 = ot.CorrelationMatrix(chainDim)  # precision matrix
Q0_inv = ot.CorrelationMatrix(chainDim)  # variance matrix
for i in range(chainDim):
    Q0_inv[i, i] = sigma0[i] * sigma0[i]
    Q0[i, i] = 1.0 / Q0_inv[i, i]
print("Q0=", Q0)

mu0 = [0.0] * chainDim
prior = ot.Normal(mu0, Q0_inv)  # x0 ~ N(mu0, sigma0)
print("x~", prior)

# start from te mean x0=(0.,0.,0.)
print("x0=", mu0)

# conditional distribution y~N(z, 1.0)
conditional = ot.Normal()
print("y~", conditional)

# create a gibbs sampler
mh_coll = [
    ot.RandomWalkMetropolisHastings(prior, mu0, instrumental, [i])
    for i in range(chainDim)
]
for mh in mh_coll:
    mh.setLikelihood(conditional, y_obs, linkFunction, p)
sampler = ot.Gibbs(mh_coll)

# get a realization
realization = sampler.getRealization()
print("y1=", realization)

# try to generate a sample
sampleSize = 10000
sample = sampler.getSample(sampleSize)

x_mu = sample.computeMean()
x_sigma = sample.computeStandardDeviation()

# compute covariance
x_cov = sample.computeCovariance()
P = ot.Matrix(obsSize, chainDim)
for i in range(obsSize):
    for j in range(chainDim):
        P[i, j] = p[i, j]
Qn = P.transpose() * P + Q0
Qn_inv = ot.SquareMatrix(chainDim)
for j in range(chainDim):
    I_j = [0] * chainDim
    I_j[j] = 1.0
    Qn_inv_j = Qn.solveLinearSystem(I_j)
    for i in range(chainDim):
        Qn_inv[i, j] = Qn_inv_j[i]

sigma_exp = [0] * chainDim
for i in range(chainDim):
    sigma_exp[i] = m.sqrt(Qn_inv[i, i])
y_vec = [0] * obsSize
for i in range(obsSize):
    y_vec[i] = y_obs[i, 0]

x_emp = Qn.solveLinearSystem(P.transpose() * y_vec)
mu_exp = Qn.solveLinearSystem((P.transpose() * P) * x_emp + ot.Matrix(Q0) * mu0)

print("sample mean=", x_mu)
print("expected mean=", mu_exp)
ott.assert_almost_equal(x_mu, mu_exp, 1e-1, 1e-1)

print("covariance=", x_cov)
print("expected covariance=", Qn_inv)
ott.assert_almost_equal(x_cov, Qn_inv, 1e-1, 2e-1)



import openturns as ot
import unittest
import openturns.testing as ott

ot.TESTPREAMBLE()


class CheckKarhunenLoeveValidation(unittest.TestCase):
    def test_KarhunenLoeveValidation(self):
        # Create the KL result
        numberOfVertices = 20
        interval = ot.Interval(-1.0, 1.0)
        mesh = ot.IntervalMesher([numberOfVertices - 1]).build(interval)
        covariance = ot.SquaredExponential()
        process = ot.GaussianProcess(covariance, mesh)
        sampleSize = 100
        processSample = process.getSample(sampleSize)
        threshold = 1.0e-7
        algo = ot.KarhunenLoeveSVDAlgorithm(processSample, threshold)
        algo.run()
        klresult = algo.getResult()
        # Create validation
        validation = ot.KarhunenLoeveValidation(processSample, klresult)
        # Check residuals
        residualProcessSample = validation.computeResidual()
        assert type(residualProcessSample) is ot.ProcessSample
        # Check standard deviation
        residualSigmaField = validation.computeResidualStandardDeviation()
        ot.Sample(numberOfVertices, 1)
        # ott.assert_almost_equal(residualSigmaField, exact)
        # Check mean
        residualMean = validation.computeResidualMean()
        ot.Sample(numberOfVertices, 1)
        # ott.assert_almost_equal(residualMean, exact)
        # Check graph
        graph0 = validation.drawValidation()
        graph1 = residualProcessSample.drawMarginal(0)
        graph2 = residualMean.drawMarginal(0)
        graph3 = residualSigmaField.drawMarginal(0)
        graph4 = validation.drawObservationWeight(0)
        graph5 = validation.drawObservationQuality()
        if 0:
            from openturns.viewer import View

            View(graph0).save("validation1.png")
            View(graph1).save("validation1-residual.png")
            View(graph2).save("validation1-residual-mean.png")
            View(graph3).save("validation1-residual-stddev.png")
            View(graph4).save("validation1-indiv-weight.png")
            View(graph5).save("validation1-indiv-quality.png")

    def test_KarhunenLoeveValidationMultidimensional(self):
        # Create the KL result
        numberOfVertices = 20
        interval = ot.Interval(-1.0, 1.0)
        mesh = ot.IntervalMesher([numberOfVertices - 1]).build(interval)
        outputDimension = 2
        univariateCovariance = ot.SquaredExponential()
        covarianceCollection = [univariateCovariance] * outputDimension
        multivariateCovariance = ot.TensorizedCovarianceModel(covarianceCollection)
        process = ot.GaussianProcess(multivariateCovariance, mesh)
        sampleSize = 100
        sampleSize = 10
        processSample = process.getSample(sampleSize)
        threshold = 1.0e-7
        algo = ot.KarhunenLoeveSVDAlgorithm(processSample, threshold)
        algo.run()
        klresult = algo.getResult()
        # Create the validation
        validation = ot.KarhunenLoeveValidation(processSample, klresult)
        # Check residuals
        residualProcessSample = validation.computeResidual()
        assert type(residualProcessSample) is ot.ProcessSample
        # Check standard deviation
        residualSigmaField = validation.computeResidualStandardDeviation()
        zeroSample = ot.Sample(numberOfVertices, outputDimension)
        ott.assert_almost_equal(residualSigmaField, zeroSample)
        # Check graph
        graph = validation.drawValidation()
        if False:
            from openturns.viewer import View

            View(graph).save("validation2.png")


if __name__ == "__main__":
    unittest.main()



"""
Link Pandas and OpenTURNS
=========================
"""

# sphinx_gallery_thumbnail_path = '_static/pandas.png'

# %%
# In this example we are going to explore interaction with Pandas data analysis tool.

# %%
import openturns as ot


# %%
# Create a sample from a 3-d Normal distribution
sample = ot.Normal(3).getSample(10)

# %%
# Create a DataFrame from a Sample
df = sample.asDataFrame()
df.describe()

# %%
# Create a Sample from a DataFrame
sample2 = ot.Sample.BuildFromDataFrame(df)
sample2



#! /usr/bin/env python

import openturns as ot
import math as m

ot.TESTPREAMBLE()


def printSample(name, sample):
    oss = name + "=["
    for counter in range(sample.getSize()):
        if counter != 0:
            oss += ";"
        oss += "["
        point = sample[counter]
        for coordinate in range(point.getDimension()):
            if coordinate != 0:
                oss += ","
            oss += "%.6f" % point[coordinate]
        oss += "]"
    oss += "]"
    return oss


# We create a numerical math function
myFunction = ot.SymbolicFunction(["x1", "x2"], ["(x1-0.3)*(x1-0.3)-x2*x2"])

dim = myFunction.getInputDimension()
# We create a normal distribution point of dimension 1
mean = ot.Point(dim, 0.0)
sigma = ot.Point(dim, 1.0)
R = ot.IdentityMatrix(dim)
myDistribution = ot.Normal(mean, sigma, R)

# We create a 'usual' RandomVector from the Distribution
vect = ot.RandomVector(myDistribution)

# We create a composite random vector
output = ot.CompositeRandomVector(myFunction, vect)

# We create a StandardEvent from this RandomVector  : RandomVector > 0.0
seuil = 10
myStandardEvent = ot.StandardEvent(output, ot.Greater(), seuil)

# We create the design point
designPoint = ot.Point(dim, 0.0)
C = 0.3
designPoint[0] = -m.sqrt(seuil) + C

# We create the "second" design point
pseudoDesignPoint = ot.Point(dim, 0.0)
pseudoDesignPoint[0] = m.sqrt(seuil) + C

importanceLevel = 0.01
accuracyLevel = 2
confidenceLevel = 0.999999

myTest = ot.StrongMaximumTest(
    myStandardEvent, designPoint, importanceLevel, accuracyLevel, confidenceLevel
)
print("myTest=", myTest)
myTest.run()
print("Beta = %.6f" % designPoint.norm())
print(
    "Discretised sphere radius = %.6f"
    % (designPoint.norm() * (1 + myTest.getAccuracyLevel() * myTest.getDeltaEpsilon()))
)
print("PointNumber = ", myTest.getPointNumber())
print("DesignPointVicinity Angle (rad)= %.6f" % m.acos(myTest.getDesignPointVicinity()))
print(
    "DesignPointVicinity Angle (deg)= %.6f"
    % (m.acos(myTest.getDesignPointVicinity()) * 180 / m.pi)
)
print(
    "Near Design Point Verifying Event Points Number = ",
    (myTest.getNearDesignPointVerifyingEventPoints()).getSize(),
)
print(
    "Near Design Point Violating Event Points Number = ",
    (myTest.getNearDesignPointViolatingEventPoints()).getSize(),
)
print(
    "Far Design Point Verifying Event Points Number = ",
    (myTest.getFarDesignPointVerifyingEventPoints()).getSize(),
)
print(
    "Far Design Point Violating Event Points Number = ",
    (myTest.getFarDesignPointViolatingEventPoints()).getSize(),
)
print("//////")

# parameters of the test
print("importanceLevel=%.6f" % myTest.getImportanceLevel())
print("accuracyLevel=%.6f" % myTest.getAccuracyLevel())
print("confidenceLevel=%.6f" % myTest.getConfidenceLevel())

# design point coordinates
print(
    "standardSpaceDesignPoint=[%.6f" % myTest.getStandardSpaceDesignPoint()[0],
    ",%.6f" % myTest.getStandardSpaceDesignPoint()[1],
    "]",
)

# pseudo design point coordinates
print(
    "pseudoStandardSpaceDesignPoint=[%.6f" % pseudoDesignPoint[0],
    ",%.6f" % pseudoDesignPoint[1],
    "]",
)

# cout of the coordinates of the points of the 4 samples
# NearDesignPointVerifyingEventPoints
print(
    printSample(
        "NearDesignPointVerifyingEventPointsSample",
        myTest.getNearDesignPointVerifyingEventPoints(),
    )
)
# NearDesignPointViolatingEventPoints
print(
    printSample(
        "NearDesignPointViolatingEventPoints",
        myTest.getNearDesignPointViolatingEventPoints(),
    )
)
# FarDesignPointVerifyingEventPoints
print(
    printSample(
        "FarDesignPointVerifyingEventPoints",
        myTest.getFarDesignPointVerifyingEventPoints(),
    )
)
# FarDesignPointViolatingEventPoints
print(
    printSample(
        "FarDesignPointViolatingEventPoints",
        myTest.getFarDesignPointViolatingEventPoints(),
    )
)



#! /usr/bin/env python

import openturns as ot

# Default constructor
flat1 = ot.IndicesCollection()
print("flat1=", flat1)

# Constructor with Collection<Indices>
coll2 = []
ind1 = ot.Indices(1)
ind4 = ot.Indices(4)
ind4.fill(8)
ind3 = ot.Indices(3)
ind3.fill(3)
coll2.append(ind1)
coll2.append(ind4)
coll2.append(ind3)
flat2 = ot.IndicesCollection(coll2)
print("flat2=", flat2)

# Copy constructor
flat3 = ot.IndicesCollection(flat2)
print("flat3=", flat3)

# Strided
coll4 = []
coll4.append(ind4)
coll4.append(ind4)
coll4.append(ind4)
strided1 = ot.IndicesCollection(coll4)
print("strided1=", strided1)

# Constructor with dimensions and values
full = ot.Indices(16)
full.fill()
strided2 = ot.IndicesCollection(8, 2, full)
print("strided2=", strided2)

# slicing
print("strided2[2] = ", strided2[2])
strided2[2] = [1, 3]
print("strided2 = ", strided2)
try:
    strided2[2] = [1, 2, 3]
    print("Should throw")
except TypeError:
    pass

# loop
for x in strided2:
    print(x)

try:
    ot.IndicesCollection([["zou"]])
except Exception:
    print("ok")



"""
Create unions or intersections of events
========================================
"""

# %%
# Abstract
# --------
#
# This example illustrates system events, which are defined as unions or intersections of other events.
# We will show how to estimate their probability both with Monte-Carlo sampling
# (using the class :class:`~openturns.ProbabilitySimulationAlgorithm`)
# and with a first order approximation (using the class :class:`~openturns.SystemFORM`).
#
import openturns as ot
import openturns.viewer as otv

# %%
# **Intersection**
#
# The event defined as the intersection of several events is realized when all sub-events occurs:
#
# .. math::
#    E_{sys} = \bigcap_{i=1}^N E_i
#
# **Union**
#
# The event defined as the union of several events is realized when at least one sub-event occurs:
#
# .. math::
#    E_{sys} = \bigcup_{i=1}^N E_i
#


# %%
# We consider a bivariate standard Gaussian random vector `X = (X_1, X_2)`.
dim = 2
distribution = ot.Normal(dim)
X = ot.RandomVector(distribution)


# %%
# We want to estimate the probability given by
#
# .. math::
#
#    P  = \mathbb{E}[\mathbf{1}_{\mathrm{Event}}(X_1, X_2)].
#
# We now build several events using intersections and unions.
#

# %%
# We consider three functions `f1`, `f2` and `f3` :
f1 = ot.SymbolicFunction(["x0", "x1"], ["x0"])
f2 = ot.SymbolicFunction(["x0", "x1"], ["x1"])
f3 = ot.SymbolicFunction(["x0", "x1"], ["x0+x1"])

# %%
# We build :class:`~openturns.CompositeRandomVector` from these functions and the initial distribution.
Y1 = ot.CompositeRandomVector(f1, X)
Y2 = ot.CompositeRandomVector(f2, X)
Y3 = ot.CompositeRandomVector(f3, X)

# %%
# We define three basic events :math:`E_1=\{(x_0,x_1)~:~x_0 < 0 \}`, :math:`E_2=\{(x_0,x_1)~:~x_1 > 0 \}` and :math:`E_3=\{(x_0,x_1)~:~x_0+x_1>0 \}`.
e1 = ot.ThresholdEvent(Y1, ot.Less(), 0.0)
e2 = ot.ThresholdEvent(Y2, ot.Greater(), 0.0)
e3 = ot.ThresholdEvent(Y3, ot.Greater(), 0.0)

# %%
# The restriction of the domain :math:`E_1` to :math:`[-4,4] \times [-4, 4]` is the grey area.
myGraph = ot.Graph(r"Representation of the event $E_1$", r"$x_1$", r"$x_2$")
data = [[-4, -4], [0, -4], [0, 4], [-4, 4]]
myPolygon = ot.Polygon(data)
myPolygon.setColor("grey")
myPolygon.setEdgeColor("black")
myGraph.add(myPolygon)
view = otv.View(myGraph)
axes = view.getAxes()
_ = axes[0].set_xlim(-4.0, 4.0)
_ = axes[0].set_ylim(-4.0, 4.0)


# %%
# The restriction of the domain :math:`E_2` to :math:`[-4,4] \times [-4, 4]` is the grey area.
myGraph = ot.Graph(r"Representation of the event $E_2$", r"$x_1$", r"$x_2$")
data = [[-4, 0], [4, 0], [4, 4], [-4, 4]]
myPolygon = ot.Polygon(data)
myPolygon.setColor("grey")
myPolygon.setEdgeColor("black")
myGraph.add(myPolygon)
view = otv.View(myGraph)
axes = view.getAxes()
_ = axes[0].set_xlim(-4.0, 4.0)
_ = axes[0].set_ylim(-4.0, 4.0)

# %%
# The restriction of the domain :math:`E_3` to :math:`[-4,4] \times [-4, 4]` is the grey area.
myGraph = ot.Graph(r"Representation of the event $E_3$", r"$x_1$", r"$x_2$")
data = [[-4, 4], [4, -4], [4, 4]]
myPolygon = ot.Polygon(data)
myPolygon.setColor("grey")
myPolygon.setEdgeColor("black")
myGraph.add(myPolygon)
view = otv.View(myGraph)
axes = view.getAxes()
_ = axes[0].set_xlim(-4.0, 4.0)
_ = axes[0].set_ylim(-4.0, 4.0)

# %%
# We can define the intersection :math:`E_4 = E_1 \bigcap E_2`: that is the upper left quadrant.
e4 = ot.IntersectionEvent([e1, e2])

# %%
# The restriction of the domain :math:`E_4` to :math:`[-4,4] \times [-4, 4]` is the grey area.
myGraph = ot.Graph(
    r"Representation of the event $E_4  = E_1 \bigcap E_2$",
    r"$x_1$",
    r"$x_2$",
)
data = [[-4, 0], [0, 0], [0, 4], [-4, 4]]
myPolygon = ot.Polygon(data)
myPolygon.setColor("grey")
myPolygon.setEdgeColor("black")
myGraph.add(myPolygon)
view = otv.View(myGraph)
axes = view.getAxes()
_ = axes[0].set_xlim(-4.0, 4.0)
_ = axes[0].set_ylim(-4.0, 4.0)

# %%
# The probability of that event is :math:`P_{E_4} = 1/4`. A basic estimator is:
print("Probability of e4 : %.4f" % e4.getSample(10000).computeMean()[0])

# %%
# We define the union :math:`E_5 = E1 \bigcup E_2`. It is the whole plan without the lower right quadrant.
e5 = ot.UnionEvent([e1, e2])

# %%
# The restriction of the domain :math:`E_5` to :math:`[-4,4] \times [-4, 4]` is the grey area.
myGraph = ot.Graph(
    r"Representation of the event $E_5  = E_1 \bigcup E_2$",
    r"$x_1$",
    r"$x_2$",
)
data = [[-4, -4], [0, -4], [0, 0], [4, 0], [4, 4], [-4, 4]]
myPolygon = ot.Polygon(data)
myPolygon.setColor("grey")
myPolygon.setEdgeColor("black")
myGraph.add(myPolygon)
view = otv.View(myGraph)
axes = view.getAxes()
_ = axes[0].set_xlim(-4.0, 4.0)
_ = axes[0].set_ylim(-4.0, 4.0)

# %%
# The probability of that event is :math:`P_{E_5} = 3/4`. A basic estimator is:
print("Probability of e5 : %.4f" % e5.getSample(10000).computeMean()[0])

# %%
# It supports recursion. Let's define :math:`E_6 = E_1 \bigcup (E_2 \bigcap E_3)`.
e6 = ot.UnionEvent([e1, ot.IntersectionEvent([e2, e3])])


# %%
# First we draw the domain :math:`E_6 = E_1 \bigcup (E_2 \bigcap E_3)` :
myGraph = ot.Graph(
    r"Representation of the event $E_2 \bigcap E_3 $", r"$x_1$", r"$x_2$"
)
data = [[-4, 4], [0, 0], [4, 0], [4, 4]]
myPolygon = ot.Polygon(data)
myPolygon.setColor("grey")
myPolygon.setEdgeColor("black")
myGraph.add(myPolygon)
view = otv.View(myGraph)
axes = view.getAxes()
_ = axes[0].set_xlim(-4.0, 4.0)
_ = axes[0].set_ylim(-4.0, 4.0)

# %%
# From the previous figures we easily deduce that the event :math:`E_6 = E_1 \bigcup (E_2 \bigcap E_3)`
# is the event :math:`E_5` and the probability is :math:`P_{E_6} = 3/4`. We can use a basic estimator and get :
print("Probability of e6 : %.4f" % e6.getSample(10000).computeMean()[0])


# %%
# Usage with a Monte-Carlo algorithm
# ----------------------------------
#
# Of course, we can use simulation algorithms with this kind of events.

# %%
# We set up a :class:`~openturns.MonteCarloExperiment` and a :class:`~openturns.ProbabilitySimulationAlgorithm` on the event :math:`E_6`.
experiment = ot.MonteCarloExperiment()
algo = ot.ProbabilitySimulationAlgorithm(e6, experiment)
algo.setMaximumOuterSampling(2500)
algo.setBlockSize(4)
algo.setMaximumCoefficientOfVariation(-1.0)
algo.run()

# %%
# We retrieve the results and display the approximate probability and a confidence interval:
result = algo.getResult()
prb = result.getProbabilityEstimate()
print("Probability of e6 through MC : %.4f" % prb)
ci95 = result.getProbabilityDistribution().computeBilateralConfidenceInterval(0.95)
print("Confidence interval (0.95) = ", ci95)


# %%
# Usage with SystemFORM
# ---------------------
#
# The :class:`~openturns.SystemFORM` class implements an approximation method suitable for system events.
# The event must be in its disjunctive normal form (union of intersections, or a single intersection).

# %%
# For system events, we always have to use the same root cause (input distribution). Here we use input variables with a
# normal distribution specified by its mean, standard deviation and correlation matrix.
#
dim = 5
mean = [200.0] * dim
mean[-1] = 60
mean[-2] = 60
sigma = [30.0] * dim
sigma[-1] = 15.0
R = ot.CorrelationMatrix(dim)
for i in range(dim):
    for j in range(i):
        R[i, j] = 0.5
dist = ot.Normal(mean, sigma, R)


# %%
# As usual we create a :class:`~openturns.RandomVector` out of the input distribution.
X = ot.RandomVector(dist)

# %%
# We define the leaf events thanks to :class:`~openturns.SymbolicFunction`.
inputs = ["M1", "M2", "M3", "M4", "M5"]
e0 = ot.ThresholdEvent(
    ot.CompositeRandomVector(ot.SymbolicFunction(inputs, ["M1-M2+M4"]), X),
    ot.Less(),
    0.0,
)
e1 = ot.ThresholdEvent(
    ot.CompositeRandomVector(ot.SymbolicFunction(inputs, ["M2+2*M3-M4"]), X),
    ot.Less(),
    0.0,
)
e2 = ot.ThresholdEvent(
    ot.CompositeRandomVector(ot.SymbolicFunction(inputs, ["2*M3-2*M4-M5"]), X),
    ot.Less(),
    0.0,
)
e3 = ot.ThresholdEvent(
    ot.CompositeRandomVector(ot.SymbolicFunction(inputs, ["-(M1+M2+M4+M5-5*10.0)"]), X),
    ot.Less(),
    0.0,
)
e4 = ot.ThresholdEvent(
    ot.CompositeRandomVector(ot.SymbolicFunction(inputs, ["-(M2+2*M3+M4-5*40.0)"]), X),
    ot.Less(),
    0.0,
)

# %%
# We consider a system event in disjunctive normal form (union of intersections):
event = ot.UnionEvent(
    [ot.IntersectionEvent([e0, e3, e4]), ot.IntersectionEvent([e2, e3, e4])]
)

# %%
# We can estimate the probability of the event with basic sampling.
print("Probability of the event : %.4f" % event.getSample(10000).computeMean()[0])

# %%
# We can also run a :class:`~openturns.SystemFORM` algorithm to estimate the probability differently.

# %%
# We first set up a solver to find the design point.
solver = ot.AbdoRackwitz()
solver.setMaximumIterationNumber(1000)
solver.setMaximumAbsoluteError(1.0e-3)
solver.setMaximumRelativeError(1.0e-3)
solver.setMaximumResidualError(1.0e-3)
solver.setMaximumConstraintError(1.0e-3)

# %%
# We build the :class:`~openturns.SystemFORM` algorithm from the solver, the event and a starting point (here the mean) and then run the algorithm.
solver.setStartingPoint(mean)
algo = ot.SystemFORM(solver, event)
algo.run()

# %%
# We store the result and display the probability.
result = algo.getResult()
prbSystemFORM = result.getEventProbability()
print("Probability of the event (SystemFORM) : %.4f" % prbSystemFORM)

# %%
# Display all figures
otv.View.ShowAll()



#! /usr/bin/env python

import copy
import openturns as ot
import openturns.testing as ott

ot.TESTPREAMBLE()
mesh = ot.RegularGrid(0, 1, 4)
vals1 = [
    ot.Sample([[1, 2, 3], [2, 3, 4], [3, 4, 5], [4, 5, 6]]),
    ot.Sample([[5, 6, 7], [6, 7, 8], [7, 8, 9], [8, 9, 10]]),
]
vals2 = [
    ot.Sample([[0, 1, 2], [1, 2, 3], [2, 3, 4], [3, 4, 5]]),
    ot.Sample([[4, 5, 6], [5, 6, 7], [6, 7, 8], [7, 8, 9]]),
]

ps1 = ot.ProcessSample(mesh, vals1)
ps2 = ot.ProcessSample(mesh, vals2)

f1 = ot.Field(mesh, vals1[0])
f2 = ot.Field(mesh, vals2[0])

s = vals1[0]
p = ot.Point((1, 2, 3))

# ProcessSample operations
sumVals = [vals1[0] + vals2[0], vals1[1] + vals2[1]]
diffVals = [vals1[0] - vals2[0], vals1[1] - vals2[1]]

psSum = ot.ProcessSample(mesh, sumVals)
psDiff = ot.ProcessSample(mesh, diffVals)

ott.assert_almost_equal(ps1 + ps2, psSum)
ott.assert_almost_equal(ps1 - ps2, psDiff)
psInPlaceSum = copy.deepcopy(ps1)
psInPlaceSum += ps2
ott.assert_almost_equal(psInPlaceSum, psSum)
psInPlaceSum -= ps2
ott.assert_almost_equal(psInPlaceSum, ps1)

# ProcessSample  and Field operations
sumVals = [vals1[0] + vals1[0], vals1[1] + vals1[0]]
diffVals = [vals1[0] - vals1[0], vals1[1] - vals1[0]]

psSum = ot.ProcessSample(mesh, sumVals)
psDiff = ot.ProcessSample(mesh, diffVals)

ott.assert_almost_equal(ps1 + f1, psSum)
ott.assert_almost_equal(ps1 - f1, psDiff)
psInPlaceSum = copy.deepcopy(ps1)
psInPlaceSum += f1
ott.assert_almost_equal(psInPlaceSum, psSum)
psInPlaceSum -= f1
ott.assert_almost_equal(psInPlaceSum, ps1)

# ProcessSample and Sample operations
ott.assert_almost_equal(ps1 + s, psSum)
ott.assert_almost_equal(ps1 - s, psDiff)
psInPlaceSum = copy.deepcopy(ps1)
psInPlaceSum += s
ott.assert_almost_equal(psInPlaceSum, psSum)
psInPlaceSum -= s
ott.assert_almost_equal(psInPlaceSum, ps1)

# ProcessSample and point operations
sumVals = [vals1[0] + p, vals1[1] + p]
diffVals = [vals1[0] - p, vals1[1] - p]

psSum = ot.ProcessSample(mesh, sumVals)
psDiff = ot.ProcessSample(mesh, diffVals)

ott.assert_almost_equal(ps1 + p, psSum)
ott.assert_almost_equal(ps1 - p, psDiff)
psInPlaceSum = copy.deepcopy(ps1)
psInPlaceSum += p
ott.assert_almost_equal(psInPlaceSum, psSum)
psInPlaceSum -= p
ott.assert_almost_equal(psInPlaceSum, ps1)
